!*******************************************
!  Estimation of school 28
!*******************************************


!******************************************
!* module: globvar
!* date: 05/26/2009
!* last modified: 1/14/2012
!* author: Angelo Mele
!* description: contains the global vbls
!******************************************

module globvar
implicit none

! contains current network in main program
integer, allocatable:: g(:,:)  ! network matrix
real(8), allocatable:: tobs(:)

! contains data on individual attributes
real(8), allocatable::x(:,:)


! contains network used in simulations
integer, allocatable:: gsim(:,:)  ! network matrix
real(8), allocatable::tsim(:) ! simulated stats


integer totsim

! contains parameters for sequential estimation 
integer, allocatable::schall(:) ! vector containing the ids of schools to estimate
integer, allocatable::nall(:) ! vector containin total students in each school
integer, allocatable::nnzaall(:) ! contains the # of links in each school
integer, allocatable::nnzsim(:) ! contains the # of links for current network for phase2 robmon
real(8), allocatable::shareall(:,:) ! contains the shares of each racial group in each school
integer sid1,sid2   ! indicator of initial school and final school in schall
!integer, allocatable::ir(:),jc(:) ! network in coo format


end module globvar




!************************************
!* program: main
!* date: 1/14/2012
!* last modified: 2/05/2012
!* author: Angelo Mele
!* description: generates an erdos-renyi network to be used as data.
!*              estimates the posterior using exact metropolis-hastings.
!*              estimates the posterior using approx exchange algorithm, 
!*              with different R (number of network simulations) 
!*
!************************************


program main

use globvar
implicit none


!**** general parameters ****
integer n !number of agents
integer pp(3) !parameters to estimate
integer qq !columns of x (attributes of agents)


!**** network variables ****
integer, allocatable:: dt(:,:) ! network stats (linear specifications)

!**** estimation/simulation parameters ****
real(8), allocatable:: a0(:) !parameters for simulation
integer n1 ! number of network simulations
integer skip ! number of skipped networks in simulations
integer, dimension(1):: seed1 !seed for random number generation
real(8) qup ! prob of increasing temperature
real(8), allocatable::tsamp(:,:) ! sampled stats
real(8), allocatable::ktemps(:) ! sample of temperatures
integer temp1 ! number of temperatures

!**** working variables
integer i,j ! counting for loops
real(8) uu ! random vbls
integer, allocatable::ir(:),jr(:) ! for matrix in coo format
integer nlink
real(8) prow,pcol ! prob of updating a column or a row of the adjacency matrix

real(8) startime, endtime

integer parsamp   ! number of parameter simulations for estimation
integer netsamp   ! number of network simulations for estimation
real(8), allocatable::step(:,:)


!**************************************************************************
! end of variables definitions
!**************************************************************************


!**** set initial parameters values ***
n = 150 ! number of players
qq = 22  ! number of variables in x
nlink = 355 ! number of links in network data


pp(1)=6 ! number of parameters, direct utility
pp(2)=12 ! number of parameters, mutual utility
pp(3)=18 ! number of parameters, indirect utility

!**** allocate memory to vectors ****
allocate(dt(pp(3),2),tobs(pp(3)),a0(pp(3))   )
allocate(x(n,qq))
allocate(g(n,n))

!**** specification of utility terms ****
!dt(1:pp(3),1) = (/1,1,1,1/)
!dt(1:pp(3),2) = (/1,9,1,1/)

dt(1:pp(3),1) = (/1,2,3,1,1,1,1,2,3,1,1,1,1,2,3,1,1,1/)
dt(1:pp(3),2) = (/1,9,9,21,32,54,1,6,6,21,22,24,1,9,9,21,32,54/)




!**** LOAD initial network
!** random numbers seed
seed1(1)=1977
i=1
!CALL RANDOM_SEED(SIZE=i)
!CALL RANDOM_SEED(PUT=seed1)
CALL RANDOM_SEED()


!**** this is to read a network file in format coo
allocate(ir(nlink),jr(nlink))
ir(:)=0
jr(:)=0
open(2,file="/hpcapp1/home/amele1/addhealth/schools/saturated/g28.txt")
do i=1,nlink
    read(2,*) ir(i),jr(i)
enddo
close(2)

!*** transform in a dense matrix
g(:,:) = 0 ! make sure you always initialize the matrix to zero
do i=1,nlink
    g(ir(i),jr(i))=1
enddo

!print*,g
deallocate(ir,jr)




!**** vectors of characteristics
x=0.0
open(2,file="/hpcapp1/home/amele1/addhealth/schools/saturated/x28.txt")
do i=1,n
    	  read(2,*) x(i,:) 
enddo
close(2)
!print*, x(1,:)
!print*, x(150,:)
!print*, sum(g)




!*** allocate memory for variables
allocate(gsim(n,n))
allocate(tsim(pp(3)))



!*** open file for writing time of computation
open(5,file='computation_sch28.txt')








!************************************************************
!* ESTIMATION WITH APPROXIMATE EXCHANGE ALGORITHM R=10000
!************************************************************



gsim = g  

!****** compute statistics of the network
call sstat(pp,n,qq,0,dt,tobs)
!print*, tobs

!*** set parameters for simulations
parsamp = 50000 !number of parameter proposals
netsamp = 10000 ! number of network simulations per parameter proposal (R)
skip =1  ! number of networks to skip for sampling
temp1 = 1 ! temperatures for network simulations (If using Simulated Tempering)
qup=0.5 ! prob of increasing temperature
prow =  0.0 ! prob of proposing a swap of a row  in the network
pcol= 0.0  ! prob of proposing a swap of a column in the network



!*** allocate memory for step of random walk
allocate(step(pp(3),pp(3)))

step(1,1)=0.5
step(2,2)=0.5
step(3,3)=0.5
step(4,4)=0.5
step(5,5)=0.5
step(6,6)=0.5
step(7,7)=0.1
step(8,8)=0.1
step(9,9)=0.1
step(10,10)=0.1
step(11,11)=0.1
step(12,12)=0.1
step(13,13)=0.1
step(14,14)=0.1
step(15,15)=0.1
step(16,16)=0.1
step(17,17)=0.1
step(18,18)=0.1


!step = step* sqrt(2.38/18)


!*** open files to use for writing output
open(1,file='parbayes_exch_sch28_R10000.txt')
open(2,file='accratio_exch_sch28_R10000.txt')


!*** note starting time for simulations
call cpu_time(startime)

!**** start simulation
call bayesest_ex(n,pp,qq,dt,skip,parsamp,netsamp,step,temp1,qup,seed1,prow,pcol)

!*** end time of simulation
call cpu_time(endtime)

!print*, '*****************'
!print*, 'Time for',parsamp,'simulations=', endtime-startime


write(5,*), 'R=', netsamp
write(5,*), 'Time for',parsamp,'simulations=', endtime-startime



stop

!**** this version of the code has the simulated network as a global variable
!**** so any subroutine or function or program can see it

!*** deallocate memory
deallocate(tsamp,ktemps)



end program main





!******** FUNCTIONS AND SUBROUTINES USED IN THE MAIN PROGRAM



!************************************
!* subroutine: dnscoo
!* version: 2.0
!* date: 05/21/2009
!* last modified: 12/05/2012
!* author: Angelo Mele
!* description: routine to convert a network matrix
!*              in dense format to a coordinate 
!*              format (coo) using global vbls
!*              (based on sparskit2 by Yousef Saad, 
!*              modified for binary matrices by Angelo Mele)
!************************************

subroutine dnscoo(n,nnz,which,ir,jc)

! DNSCOO: routine to convert a network matrix from 
!         dns format to csr format
!
! INPUT VBLS
! n    = number of agents (INTEGER)
! nnz  = number of nonzero in matrix (INTEGER)
! which = which network matrix: 0 = observed network; 1 = simulated network (INTEGER)
!
! OUTPUT VBLS
! ir   = vector with row coordinate (INTEGER,DIMENSION(nnz))
! jc   = vector with column coordinate (INTEGER,DIMENSION(nnz))
!
! WORKING VBLS
! i,j,next  = countera (INTEGER)

use globvar
implicit none

! input vbls
integer, intent(in)::n
integer, intent(in)::nnz
integer, intent(in)::which

! output vbls
integer, intent(out)::ir(nnz)
integer, intent(out)::jc(nnz)

! working vbls
integer i,j,k


if (which==0) then  ! use observed matrix
	k=1
	outer: do i=1,n
				do j=1,n
					if (k>nnz) exit outer
					if (g(i,j)==1) then
						ir(k)=i
						jc(k)=j
						k=k+1
					endif
				enddo !j
			enddo outer !i
elseif (which==1) then
	k=1
	outer2: do i=1,n
				do j=1,n
					if (k>nnz) exit outer2
					if (gsim(i,j)==1) then
						ir(k)=i
						jc(k)=j
						k=k+1
					endif
				enddo !j
			enddo outer2 !i
endif 
return
end subroutine dnscoo



!************************************
!* subroutine: sstat
!* version: 2.0
!* date: 05/22/2009
!* last modified: 12/05/2012
!* author: Angelo Mele
!* description: computes the sufficient stats
!*              given a vector of change statistics
!*              implemented with global variables
!************************************

subroutine sstat(ps,ns,qs,which,dts,ts)

! SSTAT: functions that computes ALL sufficient statistis
!        given a vector of change statistics
!
! INPUT
! ps   = vector with number of suff stats (ppu,ppm,ppv) (INTEGER(3))
! ns   = number of players (INTEGER)
! qs   = number of variables in xxx (columns of xxx), INTEGER
! which= which matrix: 0 = observed ; 1 = simulated (integer)
! dts  = vector containing the specification (INTEGER, DIMENSION(ppp,2)
! ts   = vector of sufficient statistics

use globvar
implicit none

! inpur vbls
integer, dimension(3), intent(in):: ps
integer, intent(in):: ns
integer, intent(in):: qs
integer, intent(in):: which
integer, dimension(ps(3),2), intent(in):: dts

! output vbls
real(8), dimension(ps(3)), intent(out):: ts
! working vbls
integer i,j,j1,k
real(8) dtfu,dtfm,dtfvq

ts(1:ps(3))=0.0


if (which==0) then

! statistics for u
do i=1,ns
	do j = 1,ns
        if (g(i,j)==1) then
        do k=1,ps(1)
    		ts(k) = ts(k) + dtfu(ns,dts(k,1),i,j,dts(k,2))
        enddo !k
        endif
	enddo !j
enddo !i


! statistics for m
do i=1,ns
	do j=i,ns
		if (g(i,j)==1 .AND. g(j,i)==1) then	
        do k=ps(1)+1,ps(2)
			ts(k) = ts(k) + dtfm(ns,dts(k,1),i,j,dts(k,2)) 
		enddo ! k
		endif
	enddo !j
enddo !i


! statistics for v
do i=1,ns
	do j = 1,ns
        if (g(i,j)==1) then
        do k=ps(2)+1,ps(3)
			ts(k) = ts(k) + dtfvq(ns,which,dts(k,1),i,j,dts(k,2))
        enddo !k
        endif !(gs(i,j)==1)
	enddo !j
enddo !i

elseif (which==1) then


! statistics for u
do i=1,ns
	do j = 1,ns
        if (gsim(i,j)==1) then
        do k=1,ps(1)
    		ts(k) = ts(k) + dtfu(ns,dts(k,1),i,j,dts(k,2))
        enddo !k
        endif
	enddo !j
enddo !i


! statistics for m
do i=1,ns
	do j=i,ns
		if (gsim(i,j)==1 .AND. gsim(j,i)==1) then	
        do k=ps(1)+1,ps(2)
			ts(k) = ts(k) + dtfm(ns,dts(k,1),i,j,dts(k,2)) 
		enddo ! k
		endif
	enddo !j
enddo !i


! statistics for v
do i=1,ns
	do j = 1,ns
        if (gsim(i,j)==1) then
        do k=ps(2)+1,ps(3)
			ts(k) = ts(k) + dtfvq(ns,which,dts(k,1),i,j,dts(k,2))
        enddo !k
        endif !(gs(i,j)==1)
	enddo !j
enddo !i


endif !(which)
return

end subroutine sstat




!************************************
!* function: dtfu
!* version: 2.0
!* date: 05/22/2009
!* last modified: 05/22/2013
!* author: Angelo Mele
!* description: contains the formula for the 
!*              change statistics of u_ij
!*              implemented with sparse network matrix
!************************************

function dtfu(nu,xcolu,iu,ju,tu)

! dtfu: contains the formula for all the change statistics of u_ij
!
! INPUT
! nu  = number of agents in the network (INTEGER) 
! iu  = player i (INTEGER)
! ju  = player j (INTEGER)
! xcolu = column of x matrix to use
! tu = type of statistics (INTEGER)
!       1 = single link
!       2 = difference: xu(i)-xu(j)
!       3 = absolute difference: |xu(i)-xu(j)|
!       4 = sum: xu(i)+xu(j)
!       5 = indicator: I(x(i)=0,x(j)=0)
!       6 = indicator: I(x(i)=0,x(j)=1)
!       7 = indicator: I(x(i)=1,x(j)=0)
!       8 = indicator: I(x(i)=1,x(j)=1)
!       9 = indicator: I(x(i)=x(j))
!      10 = indicator: I(x(i) .ne. x(j))
!
! NOTE: if you want to add some statistics you can do it here
!       by adding additional terms to the IF conditions


use globvar
implicit none

!**** input vbls ****
integer, intent(in)::nu
integer, intent(in):: iu,ju,tu, xcolu
!**** output vbls ****
real(8) dtfu
!**** working vbls ****
!real(8) indic
integer pppp

dtfu=0.0
if (tu==1) then
	dtfu=1.0         !**** edges ****
else if (tu==2) then
	dtfu = x(iu, xcolu) - x(ju, xcolu)       !**** difference ****
else if (tu==3) then
	dtfu = abs(x(iu, xcolu) - x(ju, xcolu))         !**** absolute difference ****
else if (tu==4) then
	dtfu = x(iu, xcolu) + x(ju, xcolu)       !**** sum ****
else if (tu==5) then
	pppp =  (x(iu, xcolu)==0)  .AND. (x(ju, xcolu)==0)      !**** indicator both zeros ****
	dtfu = pppp  
else if (tu==6) then
	pppp = (x(iu, xcolu)==0) .AND. (x(ju, xcolu)==1) 
	dtfu = pppp  
else if (tu==7) then
	pppp =  (x(iu, xcolu)==1) .AND. (x(ju, xcolu)==0)      !**** indicator one/zero ****
	dtfu = pppp  
else if (tu==8) then
	pppp = ( (x(iu, xcolu)==1) .AND. (x(ju, xcolu)==1) )     !**** indicator both ones ****
	dtfu = pppp 
else if (tu==9) then
	pppp =  (x(iu, xcolu)==x(ju, xcolu))       !**** same ****
	dtfu = pppp  
else if (tu==10) then 
	 pppp =  (x(iu, xcolu) .ne. x(ju, xcolu))      !**** different ****
	dtfu = pppp  
else if (tu==11) then
	dtfu = x(iu, xcolu)*x(ju, xcolu)        !**** product ****
else if (tu==12) then
	dtfu = x(iu, xcolu)             !**** x_i ****
else if (tu==13) then
	dtfu = x(ju, xcolu)            !**** x_j ****

! **** DIFFERENTIAL HOMOPHILY ***
else if (tu==21) then
	pppp =  (x(iu, xcolu)==1) .AND. (x(ju, xcolu)==1)
	dtfu = pppp  
else if (tu==22) then
	pppp =   (x(iu, xcolu)==1) .AND. (x(ju, xcolu)==2)
	dtfu = pppp  
else if (tu==23) then
	pppp =   (x(iu, xcolu)==1) .AND. (x(ju, xcolu)==3)
	dtfu = pppp  
else if (tu==24) then
	pppp =   (x(iu, xcolu)==1) .AND. (x(ju, xcolu)==4)
	dtfu = pppp  
else if (tu==25) then
	pppp =   (x(iu, xcolu)==1) .AND. (x(ju, xcolu)==5)
	dtfu = pppp  
else if (tu==31) then
	pppp =  (x(iu, xcolu)==2) .AND. (x(ju, xcolu)==1)
	dtfu = pppp  
else if (tu==32) then
	pppp =   (x(iu, xcolu)==2) .AND. (x(ju, xcolu)==2)
	dtfu = pppp  
else if (tu==33) then
	pppp =   (x(iu, xcolu)==2) .AND. (x(ju, xcolu)==3)
	dtfu = pppp  
else if (tu==34) then
	pppp =  (x(iu, xcolu)==2) .AND. (x(ju, xcolu)==4)
	dtfu = pppp  
else if (tu==35) then
	pppp  = (x(iu, xcolu)==2) .AND. (x(ju, xcolu)==5)
	dtfu = pppp  
else if (tu==41) then
	pppp  = (x(iu, xcolu)==3) .AND. (x(ju, xcolu)==1)
	dtfu = pppp  
else if (tu==42) then
	pppp  = (x(iu, xcolu)==3) .AND. (x(ju, xcolu)==2)
	dtfu = pppp  
else if (tu==43) then
	pppp  = (x(iu, xcolu)==3) .AND. (x(ju, xcolu)==3)
	dtfu = pppp  
else if (tu==44) then
	pppp  = (x(iu, xcolu)==3) .AND. (x(ju, xcolu)==4)
	dtfu = pppp  
else if (tu==45) then
	pppp  = (x(iu, xcolu)==3) .AND. (x(ju, xcolu)==5)
	dtfu = pppp  
else if (tu==51) then
	pppp  = (x(iu, xcolu)==4) .AND. (x(ju, xcolu)==1)
	dtfu = pppp  
else if (tu==52) then
	pppp  = (x(iu, xcolu)==4) .AND. (x(ju, xcolu)==2)
	dtfu = pppp  
else if (tu==53) then
	pppp  = (x(iu, xcolu)==4) .AND. (x(ju, xcolu)==3)
	dtfu = pppp  
else if (tu==54) then
	pppp  = (x(iu, xcolu)==4) .AND. (x(ju, xcolu)==4)
	dtfu = pppp  
else if (tu==55) then
	pppp  = (x(iu, xcolu)==4) .AND. (x(ju, xcolu)==5)
	dtfu = pppp  
else if (tu==61) then
	pppp  = (x(iu, xcolu)==5) .AND. (x(ju, xcolu)==1)
	dtfu = pppp  
else if (tu==62) then
	pppp  = (x(iu, xcolu)==5) .AND. (x(ju, xcolu)==2)
	dtfu = pppp  
else if (tu==63) then
	pppp  = (x(iu, xcolu)==5) .AND. (x(ju, xcolu)==3)
	dtfu = pppp  
else if (tu==64) then
	pppp  = (x(iu, xcolu)==5) .AND. (x(ju, xcolu)==4)
	dtfu = pppp  
else if (tu==65) then
	pppp = (x(iu, xcolu)==5) .AND. (x(ju, xcolu)==5)
	dtfu = pppp  

endif
!print*, 'dtfu', dtfu
!dtfu = dtfu/(nu*(nu-1))
!dtfu = dtfu/nu
!dtfu = dtfu/10000.0
return
end  function dtfu





!************************************
!* function: dtfm
!* version: .0
!* date: 05/18/2009
!* last modified: 05/18/2009
!* author: Angelo Mele
!* description: contains the formula for the 
!*              change statistics of m_ij
!************************************

function dtfm(nm,xcolm,im,jm,tm)

! dtfm: contains the formula for all the change statistics of m_ij
!
! INPUT
! nm  = number of agents in the network (INTEGER) 
! xcolm =  column of matrix x to use
! im  = player i (INTEGER)
! jm  = player j (INTEGER)
! tm  = type of statistics (INTEGER)
!       1 = single link
!       2 = absolute difference: |x(i)-x(j)|
!       3 = sum: xu(i)+xu(j)
!       4 = indicator: I(x(i)=0,x(j)=0)
!       5 = indicator: I(x(i)=1,x(j)=1)
!       6 = indicator: I(x(i)=x(j))
!       7 = indicator: I(x(i) .ne. x(j))
!
! NOTE: if you want to add some statistics you can do it here
!       by adding additional terms to the IF conditions

use globvar
implicit none

! input vbls
integer, intent(in)::nm
integer, intent(in):: im,jm,tm, xcolm
! output vbls
real(8) dtfm
! working vbls
!real(8) indic
integer pppp

dtfm=0.0
if (tm==1) then
	dtfm=1.0         !**** edges ****
else if (tm==2) then
	dtfm = abs(x(im, xcolm) - x(jm, xcolm))         !**** absolute difference ****
else if (tm==3) then
	dtfm = x(im, xcolm) + x(jm, xcolm)       !**** sum ****
else if (tm==4) then
    pppp = (x(im, xcolm)==0) .AND. (x(jm, xcolm)==0)      !**** indicator both zeros ****
    dtfm = pppp
else if (tm==5) then
	pppp = (x(im, xcolm)==1) .AND. (x(jm, xcolm)==1)      !**** indicator both ones ****
    dtfm = pppp
else if (tm==6) then
	pppp = (x(im, xcolm)==x(jm, xcolm))
	!dtfm = indic(x(im, xcolm),x(jm, xcolm))     !**** same ****
    dtfm = pppp
else if (tm==7) then 
	pppp = (x(im, xcolm)/=x(jm, xcolm))
    dtfm = pppp
	!dtfm = 1.0 - indic(x(im, xcolm),x(jm, xcolm))      !**** different ****
else if (tm==8) then
	dtfm = x(im, xcolm)*x(jm, xcolm)      !**** product ****
else if (tm==9) then
	dtfm = x(im, xcolm)      !**** xi ****
else if (tm==10) then
	dtfm = x(jm, xcolm)      !**** xj ****

! **** DIFFERENTIAL HOMOPHILY ***
else if (tm==21) then
	pppp = (x(im, xcolm)==1) .AND. (x(jm, xcolm)==1)
    dtfm = pppp
else if (tm==22) then
	pppp = (x(im, xcolm)==2) .AND. (x(jm, xcolm)==2)
    dtfm = pppp
else if (tm==23) then
	pppp = (x(im, xcolm)==3) .AND. (x(jm, xcolm)==3)
    dtfm = pppp
else if (tm==24) then
	pppp = (x(im, xcolm)==4) .AND. (x(jm, xcolm)==4)
    dtfm = pppp
else if (tm==25) then
	pppp = (x(im, xcolm)==5) .AND. (x(jm, xcolm)==5)
    dtfm = pppp

endif
!dtfm = dtfm/((nm*(nm-1))/2)
!dtfm = dtfm/(nm*(nm-1))
!dtfm = dtfm/(nm/2)
!dtfm = dtfm/10000.0

return
end  function dtfm






!************************************
!* function: dtfvq
!* version: 2.0
!* date: 05/22/2009
!* last modified: 05/22/2009
!* author: Angelo Mele
!* description: contains the formula for the 
!*              change statistics for v_ij 
!*              ONLY TO COMPUTE POTENTIAL FUNCTION
!*              implemented with sparse network matrix
!************************************

function dtfvq(nv,which,xcolvq,iv,jv,tvq)

! dtfvq: contains the formula for all the change statistics
!
! INPUT
! nv   = number of agents in the network (INTEGER) 
! which =  network matrix: 0=observed, 1=simulated
! xcolvq= column of matrix x to use
! iv   = player i (INTEGER)
! jv   = player j (INTEGER)
! tv   = type of statistics (INTEGER)
!       1 = single link
!       2 = difference: x(i)-x(j)
!       3 = absolute difference: |x(i)-x(j)|
!       4 = sum: x(i)+x(j)
!       5 = indicator: I(x(i)=0,x(j)=0)
!       6 = indicator: I(x(i)=0,x(j)=1)
!       7 = indicator: I(x(i)=1,x(j)=0)
!       8 = indicator: I(x(i)=1,x(j)=1)
!       9 = indicator: I(x(i)=x(j))
!      10 = indicator: I(x(i) .ne. x(j))
!
! NOTE: if you want to add some statistics you can do it here
!       by adding additional terms to the IF conditions

use globvar
implicit none

! input vbls
integer, intent(in)::nv
integer, intent(in)::which
integer, intent(in):: iv,jv,tvq, xcolvq
! output vbls
real(8) dtfvq
! working vbls
integer i0,k0,ch
!real(8) indic
integer pppp

dtfvq=0.0

if (which==0) then

!*********************
!*    single link    *
!*********************
if (tvq==1) then
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                dtfvq = dtfvq + 1.0
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!********************
!*    difference    *
!********************
else if (tvq==2) then  
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                dtfvq = dtfvq + ( x(iv, xcolvq) - x(k0, xcolvq) )
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0


!***********************
!* absolute difference *
!***********************
else if (tvq==3) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                dtfvq = dtfvq + abs( x(iv, xcolvq) - x(k0, xcolvq) )
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!*************
!*    sum    *
!*************
else if (tvq==4) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                dtfvq = dtfvq + ( x(iv, xcolvq) + x(k0, xcolvq) )
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0
	
!*****************************
!*    Indicator zero/zero    *
!*****************************
else if (tvq==5) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==0 ) .AND. ( x(k0, xcolvq) ==0 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0
	
!*****************************
!*    Indicator zero/one     *
!*****************************
else if (tvq==6) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==0 ) .AND. ( x(k0, xcolvq) ==1 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0
	
!*****************************
!*    Indicator one/zero     *
!*****************************
else if (tvq==7) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==1 ) .AND. ( x(k0, xcolvq) ==0 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0
	
!*****************************
!*    Indicator one/one      *
!*****************************
else if (tvq==8) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==1 ) .AND. ( x(k0, xcolvq) ==1 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!*****************************
!*    Indicator x(i)=x(j)    *
!*****************************
else if (tvq==9) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)== x(k0, xcolvq) )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!*****************************
!*    Indicator x(i)/=x(j)   *
!*****************************
else if (tvq==10) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolvq) .ne. x(k0, xcolvq) )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!*****************************
!*    Product x(i)*x(k)      *
!*****************************
else if (tvq==11) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                
                dtfvq = dtfvq + ( x(iv, xcolvq)*x(k0, xcolvq) )
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!*****************************
!*     x(j)      *
!*****************************
else if (tvq==12) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                dtfvq = dtfvq + x(jv, xcolvq)
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 1/1 *
!******************************
else if (tvq==21) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==1 ) .AND. ( x(k0, xcolvq) ==1)
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 1/2 *
!******************************
else if (tvq==22) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==1 ) .AND. ( x(k0, xcolvq) ==2 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 1/3 *
!******************************
else if (tvq==23) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==1 ) .AND. ( x(k0, xcolvq) ==3 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 1/4 *
!******************************
else if (tvq==24) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==1 ) .AND. ( x(k0, xcolvq) ==4 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 1/5 *
!******************************
else if (tvq==25) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==1 ) .AND. ( x(k0, xcolvq) ==5 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 2/1 *
!******************************
else if (tvq==31) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==2 ) .AND. ( x(k0, xcolvq) ==1 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 2/2 *
!******************************
else if (tvq==32) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp =  ( x(iv, xcolvq)==2 ) .AND. ( x(k0, xcolvq) ==2 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 2/3 *
!******************************
else if (tvq==33) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==2 ) .AND. ( x(k0, xcolvq) ==3 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 2/4 *
!******************************
else if (tvq==34) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==2 ) .AND. ( x(k0, xcolvq) ==4 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 2/5 *
!******************************
else if (tvq==35) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==2 ) .AND. ( x(k0, xcolvq) ==5 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 3/1 *
!******************************
else if (tvq==41) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==3 ) .AND. ( x(k0, xcolvq) ==1 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 3/2 *
!******************************
else if (tvq==42) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==3 ) .AND. ( x(k0, xcolvq) ==2 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 3/3 *
!******************************
else if (tvq==43) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==3 ) .AND. ( x(k0, xcolvq) ==3 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 3/4 *
!******************************
else if (tvq==44) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==3 ) .AND. ( x(k0, xcolvq) ==4 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 3/5 *
!******************************
else if (tvq==45) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==3 ) .AND. ( x(k0, xcolvq) ==5 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 4/1 *
!******************************
else if (tvq==51) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==4 ) .AND. ( x(k0, xcolvq) ==1 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 4/2 *
!******************************
else if (tvq==52) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==4 ) .AND. ( x(k0, xcolvq) ==2 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 4/3 *
!******************************
else if (tvq==53) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==4 ) .AND. ( x(k0, xcolvq) ==3 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 4/4 *
!******************************
else if (tvq==54) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==4 ) .AND. ( x(k0, xcolvq) ==4 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 4/5 *
!******************************
else if (tvq==55) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp =( x(iv, xcolvq)==4 ) .AND. ( x(k0, xcolvq) ==5 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 5/1 *
!******************************
else if (tvq==61) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==5) .AND. ( x(k0, xcolvq) ==1 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 5/2 *
!******************************
else if (tvq==62) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==5 ) .AND. ( x(k0, xcolvq) ==2 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 5/3 *
!******************************
else if (tvq==63) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==5 ) .AND. ( x(k0, xcolvq) ==3 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 5/4 *
!******************************
else if (tvq==64) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==5 ) .AND. ( x(k0, xcolvq) ==4 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 5/5 *
!******************************
else if (tvq==65) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==5 ) .AND. ( x(k0, xcolvq) ==5 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0
    

else
	dtfvq=0.0
endif


elseif (which==1) then




!*********************
!*    single link    *
!*********************
if (tvq==1) then
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                dtfvq = dtfvq + 1.0
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!********************
!*    difference    *
!********************
else if (tvq==2) then  
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                dtfvq = dtfvq + ( x(iv, xcolvq) - x(k0, xcolvq) )
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0


!***********************
!* absolute difference *
!***********************
else if (tvq==3) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                dtfvq = dtfvq + abs( x(iv, xcolvq) - x(k0, xcolvq) )
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!*************
!*    sum    *
!*************
else if (tvq==4) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                dtfvq = dtfvq + ( x(iv, xcolvq) + x(k0, xcolvq) )
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0
	
!*****************************
!*    Indicator zero/zero    *
!*****************************
else if (tvq==5) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==0 ) .AND. ( x(k0, xcolvq) ==0 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0
	
!*****************************
!*    Indicator zero/one     *
!*****************************
else if (tvq==6) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==0 ) .AND. ( x(k0, xcolvq) ==1 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0
	
!*****************************
!*    Indicator one/zero     *
!*****************************
else if (tvq==7) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==1 ) .AND. ( x(k0, xcolvq) ==0 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0
	
!*****************************
!*    Indicator one/one      *
!*****************************
else if (tvq==8) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==1 ) .AND. ( x(k0, xcolvq) ==1 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!*****************************
!*    Indicator x(i)=x(j)    *
!*****************************
else if (tvq==9) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)== x(k0, xcolvq) )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!*****************************
!*    Indicator x(i)/=x(j)   *
!*****************************
else if (tvq==10) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolvq) .ne. x(k0, xcolvq) )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!*****************************
!*    Product x(i)*x(k)      *
!*****************************
else if (tvq==11) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                
                dtfvq = dtfvq + ( x(iv, xcolvq)*x(k0, xcolvq) )
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!*****************************
!*     x(j)      *
!*****************************
else if (tvq==12) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                dtfvq = dtfvq + x(jv, xcolvq)
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 1/1 *
!******************************
else if (tvq==21) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==1 ) .AND. ( x(k0, xcolvq) ==1)
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 1/2 *
!******************************
else if (tvq==22) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==1 ) .AND. ( x(k0, xcolvq) ==2 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 1/3 *
!******************************
else if (tvq==23) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==1 ) .AND. ( x(k0, xcolvq) ==3 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 1/4 *
!******************************
else if (tvq==24) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==1 ) .AND. ( x(k0, xcolvq) ==4 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 1/5 *
!******************************
else if (tvq==25) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==1 ) .AND. ( x(k0, xcolvq) ==5 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 2/1 *
!******************************
else if (tvq==31) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==2 ) .AND. ( x(k0, xcolvq) ==1 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 2/2 *
!******************************
else if (tvq==32) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp =  ( x(iv, xcolvq)==2 ) .AND. ( x(k0, xcolvq) ==2 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 2/3 *
!******************************
else if (tvq==33) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==2 ) .AND. ( x(k0, xcolvq) ==3 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 2/4 *
!******************************
else if (tvq==34) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==2 ) .AND. ( x(k0, xcolvq) ==4 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 2/5 *
!******************************
else if (tvq==35) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==2 ) .AND. ( x(k0, xcolvq) ==5 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 3/1 *
!******************************
else if (tvq==41) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==3 ) .AND. ( x(k0, xcolvq) ==1 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 3/2 *
!******************************
else if (tvq==42) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==3 ) .AND. ( x(k0, xcolvq) ==2 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 3/3 *
!******************************
else if (tvq==43) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==3 ) .AND. ( x(k0, xcolvq) ==3 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 3/4 *
!******************************
else if (tvq==44) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==3 ) .AND. ( x(k0, xcolvq) ==4 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 3/5 *
!******************************
else if (tvq==45) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==3 ) .AND. ( x(k0, xcolvq) ==5 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 4/1 *
!******************************
else if (tvq==51) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==4 ) .AND. ( x(k0, xcolvq) ==1 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 4/2 *
!******************************
else if (tvq==52) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==4 ) .AND. ( x(k0, xcolvq) ==2 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 4/3 *
!******************************
else if (tvq==53) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==4 ) .AND. ( x(k0, xcolvq) ==3 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 4/4 *
!******************************
else if (tvq==54) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==4 ) .AND. ( x(k0, xcolvq) ==4 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 4/5 *
!******************************
else if (tvq==55) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp =( x(iv, xcolvq)==4 ) .AND. ( x(k0, xcolvq) ==5 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 5/1 *
!******************************
else if (tvq==61) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==5) .AND. ( x(k0, xcolvq) ==1 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 5/2 *
!******************************
else if (tvq==62) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==5 ) .AND. ( x(k0, xcolvq) ==2 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 5/3 *
!******************************
else if (tvq==63) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==5 ) .AND. ( x(k0, xcolvq) ==3 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 5/4 *
!******************************
else if (tvq==64) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==5 ) .AND. ( x(k0, xcolvq) ==4 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0

!******************************
!* Differential Homophily 5/5 *
!******************************
else if (tvq==65) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolvq)==5 ) .AND. ( x(k0, xcolvq) ==5 )
                dtfvq = dtfvq + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
    enddo !k0
    
    

else
	dtfvq=0.0
endif




endif !( which )


!dtfvq = dtfvq/(nv*(nv-1)*(nv-2))
!dtfvq = dtfvq/(nv*(nv-1))
!dtfvq = dtfvq/10000.0

return
end function dtfvq






!************************************
!* function: dtfv
!* version: 2.0
!* date: 05/22/2009
!* last modified: 10/11/2012
!* author: Angelo Mele
!* description: contains the formula for the 
!*              change statistics for v_ij  
!*              implemented with sparse network matrix
!************************************

function dtfv(nv,which,xcolv,iv,jv,tvq)

! dtfvq: contains the formula for all the change statistics
!
! INPUT
! nv   = number of agents in the network (INTEGER) 
! which= network matrix: 0=observed; 1=simulated
! xcolv= column of x matrix to use
! iv   = player i (INTEGER)
! jv   = player j (INTEGER)
! tv   = type of statistics (INTEGER)
!       1 = single link
!       2 = difference: x(i)-x(j)
!       3 = absolute difference: |x(i)-x(j)|
!       4 = sum: x(i)+x(j)
!       5 = indicator: I(x(i)=0,x(j)=0)
!       6 = indicator: I(x(i)=0,x(j)=1)
!       7 = indicator: I(x(i)=1,x(j)=0)
!       8 = indicator: I(x(i)=1,x(j)=1)
!       9 = indicator: I(x(i)=x(j))
!      10 = indicator: I(x(i) .ne. x(j))
!
! NOTE: if you want to add some statistics you can do it here
!       by adding additional terms to the IF conditions

use globvar
implicit none

! input vbls
integer, intent(in)::nv
integer, intent(in)::which
integer, intent(in)::xcolv
integer, intent(in):: iv,jv,tvq
! output vbls
real(8) dtfv
! working vbls
integer i0,k0,ch
!real(8) indic
integer pppp

dtfv=0.0


if (which==0) then


!*********************
!*    single link    *
!*********************
if (tvq==1) then
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                dtfv = dtfv + 1.0
            endif !(g(jv,k0)==1)           
        endif !(k0 .ne. iv)
        
        if (k0 .ne. jv) then
            if (g(k0,iv)==1) then
                dtfv = dtfv + 1.0
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
 
    enddo !k0
	
!********************
!*    difference    *
!********************
else if (tvq==2) then  
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                dtfv = dtfv + ( x(iv, xcolv) - x(k0, xcolv) )
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)

        if (k0 .ne. jv) then
            if (g(k0,iv)==1) then
                dtfv = dtfv + ( x(k0, xcolv) - x(jv, xcolv) )
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)

    enddo !k0


!***********************
!* absolute difference *
!***********************
else if (tvq==3) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                dtfv = dtfv + abs( x(iv, xcolv) - x(k0, xcolv) )
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
        if (k0 .ne. jv) then
            if (g(k0,iv)==1) then
                dtfv = dtfv + abs( x(k0, xcolv) - x(jv, xcolv) )
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)

 
    enddo !k0

!*************
!*    sum    *
!*************
else if (tvq==4) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                dtfv = dtfv + ( x(iv, xcolv) + x(k0, xcolv) )
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (g(k0,iv)==1) then
                dtfv = dtfv + ( x(k0, xcolv) + x(jv, xcolv) )
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)

         
    enddo !k0
	
!*****************************
!*    Indicator zero/zero    *
!*****************************
else if (tvq==5) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==0 ) .AND. ( x(k0, xcolv) ==0 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (g(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==0) .AND. (x(jv, xcolv) ==0 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)

         
    enddo !k0
	
!*****************************
!*    Indicator zero/one     *
!*****************************
else if (tvq==6) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==0 ) .AND. ( x(k0, xcolv) ==1 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (g(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==0) .AND. (x(jv, xcolv) ==1 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0
	
!*****************************
!*    Indicator one/zero     *
!*****************************
else if (tvq==7) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==1 ) .AND. ( x(k0, xcolv) ==0 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (g(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==1) .AND. (x(jv, xcolv) ==0 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0
	
!*****************************
!*    Indicator one/one      *
!*****************************
else if (tvq==8) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==1 ) .AND. ( x(k0, xcolv) ==1 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (g(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==1) .AND. (x(jv, xcolv) ==1 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!*****************************
!*    Indicator x(i)=x(j)    *
!*****************************
else if (tvq==9) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolv)== x(k0, xcolv) )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (g(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==x(jv, xcolv) )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!*****************************
!*    Indicator x(i)/=x(j)   *
!*****************************
else if (tvq==10) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolv) .ne. x(k0, xcolv) )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (g(k0,iv)==1) then
                pppp = ( x(k0, xcolv) .ne. x(jv, xcolv) )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!*****************************
!*    Product x(i)*x(k)      *
!*****************************
else if (tvq==11) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                dtfv = dtfv + ( x(iv, xcolv)*x(k0, xcolv) )
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (g(k0,iv)==1) then
                dtfv = dtfv + ( x(k0, xcolv)*x(jv, xcolv) )
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!*****************************
!*     x(j)      *
!*****************************
else if (tvq==12) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                dtfv = dtfv + x(jv, xcolv)
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
        if (k0 .ne. jv) then
            if (g(k0,iv)==1) then
                dtfv = dtfv + x(k0, xcolv)
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
   enddo !k0

!******************************
!* Differential Homophily 1/1 *
!******************************
else if (tvq==21) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==1 ) .AND. ( x(k0, xcolv) ==1)
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (g(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==1) .AND. (x(jv, xcolv) ==1 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 1/2 *
!******************************
else if (tvq==22) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==1 ) .AND. ( x(k0, xcolv) ==2 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (g(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==1) .AND. (x(jv, xcolv) ==2 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 1/3 *
!******************************
else if (tvq==23) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==1 ) .AND. ( x(k0, xcolv) ==3 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (g(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==1) .AND. (x(jv, xcolv) ==3 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 1/4 *
!******************************
else if (tvq==24) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==1 ) .AND. ( x(k0, xcolv) ==4 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (g(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==1) .AND. (x(jv, xcolv) ==4 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 1/5 *
!******************************
else if (tvq==25) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==1 ) .AND. ( x(k0, xcolv) ==5 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (g(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==1) .AND. (x(jv, xcolv) ==5 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 2/1 *
!******************************
else if (tvq==31) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==2 ) .AND. ( x(k0, xcolv) ==1 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (g(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==2) .AND. (x(jv, xcolv) ==1 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 2/2 *
!******************************
else if (tvq==32) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==2 ) .AND. ( x(k0, xcolv) ==2 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (g(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==2) .AND. (x(jv, xcolv) ==2 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 2/3 *
!******************************
else if (tvq==33) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==2 ) .AND. ( x(k0, xcolv) ==3 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (g(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==2) .AND. (x(jv, xcolv) ==3 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 2/4 *
!******************************
else if (tvq==34) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==2 ) .AND. ( x(k0, xcolv) ==4 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (g(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==2) .AND. (x(jv, xcolv) ==4 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 2/5 *
!******************************
else if (tvq==35) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==2 ) .AND. ( x(k0, xcolv) ==5 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (g(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==2) .AND. (x(jv, xcolv) ==5 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 3/1 *
!******************************
else if (tvq==41) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==3 ) .AND. ( x(k0, xcolv) ==1 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (g(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==3) .AND. (x(jv, xcolv) ==1 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 3/2 *
!******************************
else if (tvq==42) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==3 ) .AND. ( x(k0, xcolv) ==2 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (g(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==3) .AND. (x(jv, xcolv) ==2 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 3/3 *
!******************************
else if (tvq==43) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==3 ) .AND. ( x(k0, xcolv) ==3 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (g(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==3) .AND. (x(jv, xcolv) ==3 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 3/4 *
!******************************
else if (tvq==44) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==3 ) .AND. ( x(k0, xcolv) ==4 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (g(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==3) .AND. (x(jv, xcolv) ==4 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 3/5 *
!******************************
else if (tvq==45) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==3 ) .AND. ( x(k0, xcolv) ==5 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (g(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==3) .AND. (x(jv, xcolv) ==5 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 4/1 *
!******************************
else if (tvq==51) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==4 ) .AND. ( x(k0, xcolv) ==1 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (g(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==4) .AND. (x(jv, xcolv) ==1 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 4/2 *
!******************************
else if (tvq==52) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==4 ) .AND. ( x(k0, xcolv) ==2 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (g(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==4) .AND. (x(jv, xcolv) ==2 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 4/3 *
!******************************
else if (tvq==53) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==4 ) .AND. ( x(k0, xcolv) ==3 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (g(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==4) .AND. (x(jv, xcolv) ==3 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 4/4 *
!******************************
else if (tvq==54) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==4 ) .AND. ( x(k0, xcolv) ==4 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (g(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==4) .AND. (x(jv, xcolv) ==4 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 4/5 *
!******************************
else if (tvq==55) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==4 ) .AND. ( x(k0, xcolv) ==5 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (g(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==4) .AND. (x(jv, xcolv) ==5 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 5/1 *
!******************************
else if (tvq==61) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==5 ) .AND. ( x(k0, xcolv) ==1 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (g(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==5) .AND. (x(jv, xcolv) ==1 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 5/2 *
!******************************
else if (tvq==62) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==5 ) .AND. ( x(k0, xcolv) ==2 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (g(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==5) .AND. (x(jv, xcolv) ==2 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 5/3 *
!******************************
else if (tvq==63) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==5 ) .AND. ( x(k0, xcolv) ==3 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (g(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==5) .AND. (x(jv, xcolv) ==3 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 5/4 *
!******************************
else if (tvq==64) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==5 ) .AND. ( x(k0, xcolv) ==4 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (g(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==5) .AND. (x(jv, xcolv) ==4 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 5/5 *
!******************************
else if (tvq==65) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (g(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==5 ) .AND. ( x(k0, xcolv) ==5 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (g(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==5) .AND. (x(jv, xcolv) ==5 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

    

else
	dtfv=0.0
endif




elseif (which==1) then



!*********************
!*    single link    *
!*********************
if (tvq==1) then
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                dtfv = dtfv + 1.0
            endif !(g(jv,k0)==1)           
        endif !(k0 .ne. iv)
        
        if (k0 .ne. jv) then
            if (gsim(k0,iv)==1) then
                dtfv = dtfv + 1.0
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
 
    enddo !k0
	
!********************
!*    difference    *
!********************
else if (tvq==2) then  
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                dtfv = dtfv + ( x(iv, xcolv) - x(k0, xcolv) )
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)

        if (k0 .ne. jv) then
            if (gsim(k0,iv)==1) then
                dtfv = dtfv + ( x(k0, xcolv) - x(jv, xcolv) )
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)

    enddo !k0


!***********************
!* absolute difference *
!***********************
else if (tvq==3) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                dtfv = dtfv + abs( x(iv, xcolv) - x(k0, xcolv) )
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
        if (k0 .ne. jv) then
            if (gsim(k0,iv)==1) then
                dtfv = dtfv + abs( x(k0, xcolv) - x(jv, xcolv) )
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)

 
    enddo !k0

!*************
!*    sum    *
!*************
else if (tvq==4) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                dtfv = dtfv + ( x(iv, xcolv) + x(k0, xcolv) )
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (gsim(k0,iv)==1) then
                dtfv = dtfv + ( x(k0, xcolv) + x(jv, xcolv) )
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)

         
    enddo !k0
	
!*****************************
!*    Indicator zero/zero    *
!*****************************
else if (tvq==5) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==0 ) .AND. ( x(k0, xcolv) ==0 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (gsim(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==0) .AND. (x(jv, xcolv) ==0 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)

         
    enddo !k0
	
!*****************************
!*    Indicator zero/one     *
!*****************************
else if (tvq==6) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==0 ) .AND. ( x(k0, xcolv) ==1 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (gsim(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==0) .AND. (x(jv, xcolv) ==1 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0
	
!*****************************
!*    Indicator one/zero     *
!*****************************
else if (tvq==7) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==1 ) .AND. ( x(k0, xcolv) ==0 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (gsim(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==1) .AND. (x(jv, xcolv) ==0 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0
	
!*****************************
!*    Indicator one/one      *
!*****************************
else if (tvq==8) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==1 ) .AND. ( x(k0, xcolv) ==1 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (gsim(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==1) .AND. (x(jv, xcolv) ==1 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!*****************************
!*    Indicator x(i)=x(j)    *
!*****************************
else if (tvq==9) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolv)== x(k0, xcolv) )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (gsim(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==x(jv, xcolv) )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!*****************************
!*    Indicator x(i)/=x(j)   *
!*****************************
else if (tvq==10) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolv) .ne. x(k0, xcolv) )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (gsim(k0,iv)==1) then
                pppp = ( x(k0, xcolv) .ne. x(jv, xcolv) )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!*****************************
!*    Product x(i)*x(k)      *
!*****************************
else if (tvq==11) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                dtfv = dtfv + ( x(iv, xcolv)*x(k0, xcolv) )
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (gsim(k0,iv)==1) then
                dtfv = dtfv + ( x(k0, xcolv)*x(jv, xcolv) )
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!*****************************
!*     x(j)      *
!*****************************
else if (tvq==12) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                dtfv = dtfv + x(jv, xcolv)
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
        if (k0 .ne. jv) then
            if (gsim(k0,iv)==1) then
                dtfv = dtfv + x(k0, xcolv)
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
   enddo !k0

!******************************
!* Differential Homophily 1/1 *
!******************************
else if (tvq==21) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==1 ) .AND. ( x(k0, xcolv) ==1)
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (gsim(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==1) .AND. (x(jv, xcolv) ==1 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 1/2 *
!******************************
else if (tvq==22) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==1 ) .AND. ( x(k0, xcolv) ==2 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (gsim(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==1) .AND. (x(jv, xcolv) ==2 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 1/3 *
!******************************
else if (tvq==23) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==1 ) .AND. ( x(k0, xcolv) ==3 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (gsim(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==1) .AND. (x(jv, xcolv) ==3 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 1/4 *
!******************************
else if (tvq==24) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==1 ) .AND. ( x(k0, xcolv) ==4 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (gsim(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==1) .AND. (x(jv, xcolv) ==4 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 1/5 *
!******************************
else if (tvq==25) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==1 ) .AND. ( x(k0, xcolv) ==5 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (gsim(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==1) .AND. (x(jv, xcolv) ==5 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 2/1 *
!******************************
else if (tvq==31) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==2 ) .AND. ( x(k0, xcolv) ==1 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (gsim(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==2) .AND. (x(jv, xcolv) ==1 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 2/2 *
!******************************
else if (tvq==32) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==2 ) .AND. ( x(k0, xcolv) ==2 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (gsim(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==2) .AND. (x(jv, xcolv) ==2 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 2/3 *
!******************************
else if (tvq==33) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==2 ) .AND. ( x(k0, xcolv) ==3 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (gsim(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==2) .AND. (x(jv, xcolv) ==3 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 2/4 *
!******************************
else if (tvq==34) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==2 ) .AND. ( x(k0, xcolv) ==4 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (gsim(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==2) .AND. (x(jv, xcolv) ==4 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 2/5 *
!******************************
else if (tvq==35) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==2 ) .AND. ( x(k0, xcolv) ==5 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (gsim(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==2) .AND. (x(jv, xcolv) ==5 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 3/1 *
!******************************
else if (tvq==41) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==3 ) .AND. ( x(k0, xcolv) ==1 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (gsim(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==3) .AND. (x(jv, xcolv) ==1 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 3/2 *
!******************************
else if (tvq==42) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==3 ) .AND. ( x(k0, xcolv) ==2 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (gsim(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==3) .AND. (x(jv, xcolv) ==2 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 3/3 *
!******************************
else if (tvq==43) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==3 ) .AND. ( x(k0, xcolv) ==3 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (gsim(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==3) .AND. (x(jv, xcolv) ==3 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 3/4 *
!******************************
else if (tvq==44) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==3 ) .AND. ( x(k0, xcolv) ==4 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (gsim(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==3) .AND. (x(jv, xcolv) ==4 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 3/5 *
!******************************
else if (tvq==45) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==3 ) .AND. ( x(k0, xcolv) ==5 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (gsim(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==3) .AND. (x(jv, xcolv) ==5 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 4/1 *
!******************************
else if (tvq==51) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==4 ) .AND. ( x(k0, xcolv) ==1 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (gsim(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==4) .AND. (x(jv, xcolv) ==1 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 4/2 *
!******************************
else if (tvq==52) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==4 ) .AND. ( x(k0, xcolv) ==2 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (gsim(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==4) .AND. (x(jv, xcolv) ==2 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 4/3 *
!******************************
else if (tvq==53) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==4 ) .AND. ( x(k0, xcolv) ==3 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (gsim(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==4) .AND. (x(jv, xcolv) ==3 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 4/4 *
!******************************
else if (tvq==54) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==4 ) .AND. ( x(k0, xcolv) ==4 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (gsim(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==4) .AND. (x(jv, xcolv) ==4 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 4/5 *
!******************************
else if (tvq==55) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==4 ) .AND. ( x(k0, xcolv) ==5 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (gsim(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==4) .AND. (x(jv, xcolv) ==5 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 5/1 *
!******************************
else if (tvq==61) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==5 ) .AND. ( x(k0, xcolv) ==1 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (gsim(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==5) .AND. (x(jv, xcolv) ==1 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 5/2 *
!******************************
else if (tvq==62) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==5 ) .AND. ( x(k0, xcolv) ==2 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (gsim(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==5) .AND. (x(jv, xcolv) ==2 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 5/3 *
!******************************
else if (tvq==63) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==5 ) .AND. ( x(k0, xcolv) ==3 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (gsim(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==5) .AND. (x(jv, xcolv) ==3 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 5/4 *
!******************************
else if (tvq==64) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==5 ) .AND. ( x(k0, xcolv) ==4 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (gsim(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==5) .AND. (x(jv, xcolv) ==4 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

!******************************
!* Differential Homophily 5/5 *
!******************************
else if (tvq==65) then          
    do k0=1,nv
        if (k0 .ne. iv) then
            if (gsim(jv,k0)==1) then
                pppp = ( x(iv, xcolv)==5 ) .AND. ( x(k0, xcolv) ==5 )
                dtfv = dtfv + pppp
            endif !(g(jv,k0)==1)
        endif !(k0 .ne. iv)
       if (k0 .ne. jv) then
            if (gsim(k0,iv)==1) then
                pppp = ( x(k0, xcolv)==5) .AND. (x(jv, xcolv) ==5 )
                dtfv = dtfv + pppp
            endif !(g(k0,iv)==1)
        endif !(k0 .ne. jv)
    enddo !k0

	
else
	dtfv=0.0
endif


endif !(which)




!dtfv = dtfv/(nv*(nv-1)*(nv-2))
!dtfv = dtfv/(nv*(nv-1))
!dtfv = dtfv/10000.0

return
end function dtfv



!!************************************
!!* subroutine: sample_ast
!!* version: 1.0
!!* date: 06/21/2009
!!* last modified: 10/11/2012
!!* author: Angelo Mele
!!* description: Samples using Accelerated Simulated
!!*              Tempering as in Li et al (2004).
!!*******************************************
!
!subroutine sample_ast(nn,qq,pp,aa,dtt,skip,sample,mtemp,qup,tout,kout,seed1)
!
!! input variables
!! nn     = number of players
!! qq     = number of variables
!! pp     = number of parameters for u,m,v
!! aa     = vector of parameters for simulations
!! dtt    = vector of statistics for simulations
!! skip   = number of samples to skip
!! sample = number of networks to sample
!! mtemp  = number of temperatures for simulated tempering
!! qup    = probability of increasing temperature in sim tempering
!! seed1  = seed for random number generation
!
!! output variables
!! tout   = vector of sampled network stats
!! kout   = vector of temperatures sampled
!
!
!
!!*********** ROUTINE STARTS HERE *************!
!
!!  library
!use globvar
!implicit none
!
!! input vbls and pars
!integer, intent(in)::nn 
!integer, intent(in)::qq
!integer, intent(in)::pp(3)
!real(8), dimension(pp(3)), intent(in)::aa
!integer, dimension(pp(3),2), intent(in)::dtt
!integer, intent(in)::skip
!integer, intent(in)::sample
!real(8) qup  
!integer, intent(in)::mtemp 
!integer, dimension(1), intent(in)::seed1
!
!
!! output vbls and pars
!real(8), dimension(pp(3),sample), intent(out)::tout
!real(8), dimension(sample), intent(out)::kout
!
!! working variables
!integer i,j,k,rr,t, kk  ! indicators for loops
!real(8), dimension(pp(3))::tsim2 ! current simulated network stats
!integer, dimension(nn,nn)::gsim2 ! current simulated networks for non-trivial steps
!integer ktemp !indicator of current temperature
!real(8) invp(mtemp) ! probability of inversion for each temperature
!real(8) temper(mtemp) ! vector with temperature levels
!real(8) rhotemp(mtemp) ! vector of pseudopriors
!real(8) rho0,nn0 ! constants for stochastic approximation of pseudopriors (see Geyer-Thompson 1995 for details)
!!integer totsim  ! maximum nummber of simulations
!real(8) ux ! random number
!real(8) odds ! odds of two networks
!real(8) dtfu, dtfm, dtfv ! network statistics functions
!real(8) dttemp(pp(3))
!
!
!! set prob of inversion
!invp(1) = 0.0 ! .01 !.01 
!do k=2,mtemp
!invp(k)=invp(k-1)+.05
!enddo
!
!! set temperatures
!temper(1)=1
!do k=2,mtemp
!	temper(k)=1 + .1*(k-1)
!enddo
!
!! set pseudopriors
!rhotemp(:)=100000.0
!rho0 = 1.0
!nn0 = 10000000.0
!
!
!
!! set initial simulated network equal to observed one
!gsim = g
!
!! compute observed network stats
!call sstat(pp,nn,qq,1,dtt,tsim)
!!tsim = tobs
!
!
!!** random numbers seed
!k=1
!CALL RANDOM_SEED(SIZE=k)
!CALL RANDOM_SEED(PUT=seed1)
!
!
!
!!**************************************************!
!!******* MAIN SIMULATION LOOP STARTS HERE *********!
!!**************************************************!
!
!tout(:,:)=0.0  ! initialize 
!rr=1   ! sample loop indicator
!ktemp =1  ! initialize temperature
!totsim = 10000000 ! max number of simulations
!
!! main loop
!do t=1, totsim
!    
!    call random_number(ux)
!    ! inversion step
!    if (ux<invp(ktemp)) then
!        gsim2 = 1-gsim
!        call sstat(pp,nn,qq,gsim2,x,dtt,tsim2)
!        odds = 0.0
!        do kk=1,pp(3)
!            odds = odds + aa(kk)*(tsim2(kk)-tsim(kk))
!        enddo ! kk
!        call random_number(ux)
!        if (log(ux)<=min(0.0,odds)) then
!            gsim = gsim2
!            tsim = tsim2
!        endif ! (log(ux)<=min(0.0,odds))
!        
!    else  
!        ! one-link per iteration 
!        call random_number(ux)
!        i = ceiling(nn*ux) ! choose player i
!        call random_number(ux)
!        j = ceiling(nn*ux) ! choose agent j
!        
!        do while (i==j)
!            call random_number(ux)
!            j = ceiling(nn*ux) ! choose agent j
!        enddo ! while loop
!        
!        call random_number(ux)
!        odds = 0.0
!        dttemp(:) = 0.0 
!        
!        ! direct utility update
!        do kk=1,pp(1)
!            dttemp(kk) = dtfu(nn,x(:,dtt(kk,1)),i,j,dtt(kk,2))
!        enddo
!        ! mutual utility update        
!        if (gsim(j,i)==1) then  ! g_ji=1 then compute mutual utility
!            do kk=pp(1)+1,pp(2)
!                dttemp(kk) = dtfm(nn,x(:,dtt(kk,1)),i,j,dtt(kk,2))
!            enddo
!        endif
!        ! indirect utility and popularity update
!        do kk=pp(2)+1,pp(3)
!            dttemp(kk) = dtfv(nn,gsim,x(:,dtt(kk,1)),i,j,dtt(kk,2))
!        enddo
!        
!       
!        
!        ! compute odds ration from the model
!        do kk=1,pp(3)
!            odds = odds + aa(kk)*dttemp(kk)
!        enddo
!        odds = odds/temper(ktemp)
!        
!    
!    
!        ! update link g_ij
!        if (gsim(i,j) ==1 ) then
!            if (odds<0) then
!                gsim(i,j)=0 
!                do kk=1,pp(3)
!                    tsim(kk) = tsim(kk) - dttemp(kk)
!                enddo
!                
!            else
!                if (log(ux)<-odds) then
!                   gsim(i,j)=0 
!                   do kk=1,pp(3)
!                        tsim(kk) = tsim(kk) - dttemp(kk)
!                    enddo
!                endif ! (log(ux)<-odds)
!                
!            endif ! (odds<0)
!            
!        elseif (gsim(i,j)==0) then
!            if (odds>=0) then
!                gsim(i,j)=1
!                do kk=1,pp(3)
!                    tsim(kk) = tsim(kk) + dttemp(kk)
!                enddo
!            else
!                if (log(ux)<odds) then
!                    gsim(i,j)=1
!                    do kk=1,pp(3)
!                        tsim(kk) = tsim(kk) + dttemp(kk)
!                    enddo
!                    
!                endif !(log(ux)<=odds)
!                
!            endif !(odds>=0)
!        endif !(gsim(i,j) ==1 )
!          
!    endif !(ux<invp(ktemp))
!
!    
!    ! sample every skip iterations
!    if (rr<=sample) then
!        if (t==skip*rr) then
!            tout(:,rr)=tsim
!            kout(rr)=ktemp
!            if (mod(rr,1000)==0) then
!                print*, 'sample = ', rr
!                print*, 'tsim = ', tsim
!            endif
!            rr=rr+1
!        endif
!    else
!        exit
!    endif
!    
!   !if (mod(rr,1000)==0) then
!   !    call sstat(pp,nn,qq,gsim,x,dtt,tobs)
!   !    print*, tsim
!   !    print*, tobs
!   ! endif
!    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!    ! introduce simulated tempering
!    
!enddo ! t
!
!end subroutine sample_ast
!
!
!
!
!
!
!
!
!
!
!
!
!!************************************
!!* subroutine: sample_ast_tnt
!!* version: 1.0
!!* date: 11/09/2012
!!* last modified:  11/09/2012
!!* author: Angelo Mele
!!* description: Samples using Accelerated Simulated
!!*              Tempering as in Li et al (2004).
!!               introduces steps a tie or not tie is selected
!!*******************************************
!
!subroutine sample_ast_tnt(nn,qq,pp,aa,dtt,skip,sample,mtemp,qup,tout,kout,seed1,pr,pc)
!
!! input variables
!! nn     = number of players
!! qq     = number of variables
!! pp     = number of parameters for u,m,v
!! aa     = vector of parameters for simulations
!! dtt    = vector of statistics for simulations
!! skip   = number of samples to skip
!! sample = number of networks to sample
!! mtemp  = number of temperatures for simulated tempering
!! qup    = probability of increasing temperature in sim tempering
!! seed1  = seed for random number generation
!! pr     = prob of changing a row of g
!! pc     = prob of changing a column of g
!
!! output variables
!! tout   = vector of sampled network stats
!! kout   = vector of temperatures sampled
!
!
!
!!*********** ROUTINE STARTS HERE *************!
!
!!  library
!use globvar
!implicit none
!
!! input vbls and pars
!integer, intent(in)::nn 
!integer, intent(in)::qq
!integer, intent(in)::pp(3)
!real(8), dimension(pp(3)), intent(in)::aa
!integer, dimension(pp(3),2), intent(in)::dtt
!integer, intent(in)::skip
!integer, intent(in)::sample
!real(8) qup  
!integer, intent(in)::mtemp 
!integer, dimension(1), intent(in)::seed1
!real(8), intent(in)::pr
!real(8), intent(in)::pc
!
!
!! output vbls and pars
!real(8), dimension(pp(3),sample), intent(out)::tout
!real(8), dimension(sample), intent(out)::kout
!
!! working variables
!integer i,j,k,rr,t, kk ,jj ! indicators for loops

!real(8), dimension(pp(3))::tsim2 ! current simulated network stats
!integer, dimension(nn,nn)::gsim2 ! current simulated networks for non-trivial steps
!integer ktemp !indicator of current temperature
!real(8) invp(mtemp) ! probability of inversion for each temperature
!real(8) temper(mtemp) ! vector with temperature levels
!real(8) rhotemp(mtemp) ! vector of pseudopriors
!real(8) rho0,nn0 ! constants for stochastic approximation of pseudopriors (see Geyer-Thompson 1995 for details)
!!integer totsim  ! maximum nummber of simulations
!real(8) ux,uux ! random number
!real(8) odds ! odds of two networks
!real(8) dtfu, dtfm, dtfv ! network statistics functions
!real(8) dttemp(pp(3))
!integer ntie ! number of ties
!integer count
!real(8) qratio 
!integer nedges !# of edges
!
!! set prob of inversion
!invp(1) = 0.0 ! .01 !.01 
!do k=2,mtemp
!invp(k)=invp(k-1)+.05
!enddo
!
!! set temperatures
!temper(1)=1
!do k=2,mtemp
!	temper(k)=1 + .1*(k-1)
!enddo
!
!! set pseudopriors
!rhotemp(:)=100000.0
!rho0 = 1.0
!nn0 = 10000000.0
!
!
!
!! set initial simulated network equal to observed one
!gsim = g
!
!! compute observed network stats
!call sstat(pp,nn,qq,g,x,dtt,tobs)
!tsim = tobs
!!call sstat(pp,nn,qq,g,x,dtt,tsim)
!
!
!!** random numbers seed
!k=1
!CALL RANDOM_SEED(SIZE=k)
!CALL RANDOM_SEED(PUT=seed1)
!
!
!
!!**************************************************!
!!******* MAIN SIMULATION LOOP STARTS HERE *********!
!!**************************************************!
!
!tout(:,:)=0.0  ! initialize 
!rr=1   ! sample loop indicator
!ktemp =1  ! initialize temperature
!totsim = 1000000000 ! max number of simulations
!nedges = sum(gsim)
!! main loop
!do t=1, totsim
!    
!    ! one link step
!    !elseif (uux <= pr+pc) then
!        call random_number(uux)
!        
!        ! one-link per iteration 
!
!        
!            
!        if (uux<0.5 .AND. nedges>0) then
!        !if (uux<0.5) then
!            !print*, 'iter=', t
!            !print*, 'tie at random'
!            !print*, 'nedges=', nedges
!        call random_number(ux)
!        i = ceiling(nn*ux) ! choose player i
!        ntie = sum(gsim(i,:))
!
!        count=0
!        do while (ntie==0) 
!                !Sprint*, 'ntie==0!!!'
!                call random_number(ux)
!                i = ceiling(nn*ux) ! choose player i 
!                ntie = sum(gsim(i,:))
!            enddo    
!        !if (ntie<=2 .OR. ntie>=(nn-1)-2) then
!            !    !if (rr>=234000) then
!            !    !print*, 'uno'
!            !    !endif
!            !    call random_number(ux)
!            !   j=ceiling(nn*ux)
!            !   
!            !else
!                !if (rr>=234000) then
!                !print*, 'due'
!                !endif
!                count=0
!                call random_number(ux)
!                jj=ceiling(ntie*ux)
!  
!    stewie:    do kk=1,nn
!                   if (count>=jj) then
!                        j=kk
!                        exit stewie
!                    else
!                        if (gsim(i,kk)==1) then
!                            if (ntie==1) then 
!                                j=kk
!                                exit stewie
!                            else
!                                
!                            count=count+1
!                            endif !(ntie==1)
!                        endif  !(g(i,kk)==1)                       
!                    endif !(count==jj)
!                enddo stewie
!            !endif !((ntie<2))    
!            ! check that it is not updating g(i,i) by mistake
!            do while (i==j)
!                !if (ntie<=2 .OR. ntie>=(nn-1)-2) then
!                !    !if (rr>=234000) then
!                     !   print*, 'tre'
!                     !   print*, i, j, jj,ntie
!                !    !endif
!                !    
!                !    call random_number(ux)
!                !    j=ceiling(nn*ux)
!                !else
!                    !if (rr>=234000) then
!                    !    print*, 'quattro'
!                    !    print*, ntie
!                    !    print*, i, j 
!                    !endif
!                    
!                    count=0
!                    call random_number(ux)
!                    jj=ceiling(ntie*ux)
!                    
!                  crik:  do kk=1,nn
!                        if (count==jj) then
!                            j=kk
!                            exit crik
!                        else
!                            if (gsim(i,kk)==1) then
!                                if (ntie==1) then 
!                                    j=kk
!                                    exit crik
!                                else
!                                    count=count+1
!                                endif
!                                
!                            endif  !(g(i,kk)==1)                       
!                        endif !(count==jj)
!                  enddo crik
!                !endif !((ntie<2))
!                
!            enddo !while (i==j)
!            
!            !qratio = log(1.0*ntie/((nn-1)-ntie))
!            !qratio = log( ( 1.0*nn*(nn-1)/tsim(1)  +1.0*tsim(1)/(nn*(nn-1))   )    / (1.0 -  tsim(1)/(nn*(nn-1))) )
!           !qratio = log(1.0*nedges/(nn*(nn-1)+nedges)) 
!            qratio = log(1.0*nedges/(nn*(nn-1)+nedges)) 
!        else
!        !print*, 'iter=', t
!        !print*, 'dyad at random'
!        !print*, 'nedges=', tsim(1)*nn*(nn-1)
!        !    
!        ! one-link per iteration 
!        call random_number(ux)
!        i = ceiling(nn*ux) ! choose player i
!        call random_number(ux)
!        j = ceiling(nn*ux) ! choose agent j
!        
!        do while (i==j)
!            call random_number(ux)
!            j = ceiling(nn*ux) ! choose agent j
!        enddo ! while loop
!        !qratio = -log(1.0*((nn-1)-ntie)/ntie)
!        !
!         !  !if (ntie<=2 .OR. ntie>=(nn-1)-2) then
!         !  !    !if (rr>=234000) then
!         !  !    !    print*, 'cinque'
!         !  !    !endif
!         !  !    !
!         !  !    call random_number(ux)
!         !  !    j=ceiling(nn*ux)
!         !  !else
!         !       !if (rr>=234000) then
!         !       !    print*, 'sei'
!         !       !endif
!         !       !
!        !        count=0
!        !        call random_number(ux)
!        !        jj=ceiling((nn-ntie)*ux)
!        ! 
!        ! dude:   do kk=1,nn
!        !            if (count==jj) then
!        !                j=kk
!        !                exit dude
!        !            else
!        !                if (gsim(i,kk)==0) then
!        !                    count=count+1
!        !                endif  !(g(i,kk)==0)                       
!        !            endif !(count==jj)
!        !        enddo dude
!        !! !   !endif !((ntie<2))
!        !    ! check that it is not updating g(i,i) by mistake
!        !    do while (i==j)
!        !! !       !if (ntie<=2 .OR. ntie>=(nn-1)-2) then
!        !! !       !    !if (rr>=234000) then
!        !! !       !    !    print*, 'sette'
!        !! !       !    !endif
!        !! !       !    !
!        !! !          call random_number(ux)
!        !! !           j=ceiling(nn*ux)
!        !! !       !else
!        !! !       
!        !! !           !if (rr>=234000) then
!        !! !           !    print*, 'otto'
!        !! !           !endif
!        !! !           
!        !            count=0
!        !           call random_number(ux)
!        !           jj=ceiling((nn-ntie)*ux)
!        ! 
!        !          crick:  do kk=1,nn
!        !             if (count==jj) then
!        !                    j=kk
!        !                    exit crick
!        !                else
!        !                    if (gsim(i,kk)==0) then
!        !                        count=count+1
!        !                    endif  !(g(i,kk)==0)                       
!        !                endif !(count==jj)
!        !          enddo crick
!        !!        !endif
!        !! !       
!        !    end do !while (i==j)
!        !! !
!        !    qratio = -log(1.0*((nn-1)-ntie)/ntie)
!            qratio=0.0
!            
!        endif !(uux<0.5)
!        
!        
!        !qratio = -log(1.0*((nn-1)-ntie)/ntie)
!        !qratio = -log(1.0*sum(gsim)/(nn*(nn-1)-sum(gsim)))
!        !qratio = -log(1.0*tsim(1)/(nn*(nn-1)-tsim(1)))
!        !qratio = log(1.0*(nn*(nn-1)-tsim(1))/(tsim(1)))
!        !qratio = -log(1.0*tsim(1)/(tsim(1)+nn*(nn-1))) 
!        !qratio =  log( ( 1.0/tsim(1) + 1.0*tsim(1)/((nn*(nn-1)) **2) )   /  ( 1.0/((nn*(nn-1))**2)  ) )
!        
!        call random_number(ux)
!        odds = 0.0
!        dttemp(:) = 0.0 
!        
!        ! direct utility update
!        do kk=1,pp(1)
!            dttemp(kk) = dtfu(nn,x(:,dtt(kk,1)),i,j,dtt(kk,2))
!        enddo
!        ! mutual utility update        
!        if (gsim(j,i)==1) then  ! g_ji=1 then compute mutual utility
!            do kk=pp(1)+1,pp(2)
!                dttemp(kk) = dtfm(nn,x(:,dtt(kk,1)),i,j,dtt(kk,2))
!            enddo
!        endif
!        ! indirect utility and popularity update
!        do kk=pp(2)+1,pp(3)
!            dttemp(kk) = dtfv(nn,gsim,x(:,dtt(kk,1)),i,j,dtt(kk,2))
!        enddo
!                
!        ! compute odds ration from the model
!        do kk=1,pp(3)
!            odds = odds + aa(kk)*dttemp(kk)
!        enddo
!        
!        odds = odds + qratio
!        odds = odds/temper(ktemp)
!        
!        ! update link g_ij
!        if (gsim(i,j) ==1 ) then
!            if (odds<0.0) then
!                gsim(i,j)=0 
!                tsim = tsim - dttemp
!                nedges=nedges-1
!                !do kk=1,pp(3)
!                !    tsim(kk) = tsim(kk) - dttemp(kk)
!                !enddo             
!            else
!                if (log(ux)<-odds) then
!                   gsim(i,j)=0 
!                   tsim = tsim - dttemp
!                   nedges=nedges-1
!                   !do kk=1,pp(3)
!                   !     tsim(kk) = tsim(kk) - dttemp(kk)
!                   !enddo
!                endif ! (log(ux)<-odds)                
!            endif ! (odds<0)  
!        endif
!        
!        if (gsim(i,j)==0) then
!            if (odds>=0.0) then
!                gsim(i,j)=1
!                !print*, 'first'
!                tsim = tsim +dttemp
!                nedges=nedges+1
!                !do kk=1,pp(3)
!                !    tsim(kk) = tsim(kk) + dttemp(kk)
!                !enddo
!            else
!                if (log(ux)<odds) then
!                    gsim(i,j)=1 
!                 !   print*, 'second'
!                    
!                    tsim = tsim + dttemp
!                    nedges=nedges+1
!                    !do kk=1,pp(3)
!                    !    tsim(kk) = tsim(kk) + dttemp(kk)
!                    !enddo                    
!                endif !(log(ux)<=odds)                
!            endif !(odds>=0)
!        endif !(gsim(i,j) ==1 )
!        
!    !call sstat(pp,nn,qq,gsim,x,dtt,tobs)
!    !print*, 'i=',i,'j=',j
!    !   print*, tsim
!    !   print*, tobs
!    !pause
!    ! sample every skip iterations
!    if (rr<=sample) then
!        if (t==skip*rr) then
!            tout(:,rr)=tsim
!            kout(rr)=ktemp
!            if (mod(rr,1000)==0) then
!                print*, 'sample = ', rr
!                print*, 'tsim = ', tsim
!            endif
!            rr=rr+1
!        endif
!    else
!        exit
!    endif
!    
!
!    
!    !if (mod(rr,1000)==0) then
!    !   call sstat(pp,nn,qq,gsim,x,dtt,tobs)
!    !   print*, tsim
!    !   print*, tobs
!    !endif
!    !
!
!    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!    ! introduce simulated tempering
!    
!enddo ! t
!
!end subroutine sample_ast_tnt
!
!
!
!



!************************************
!* subroutine: sample_ast_rc
!* version: 1.0
!* date: 11/09/2012
!* last modified:  11/09/2012
!* author: Angelo Mele
!* description: Samples using Accelerated Simulated
!*              Tempering as in Li et al (2004).
!               introduces steps where row or column
!               are modified
!*******************************************

subroutine sample_ast_rc(nn,qq,pp,aa,dtt,skip,sample,mtemp,qup,tout,kout,seed1,pr,pc)

! input variables
! nn     = number of players
! qq     = number of variables
! pp     = number of parameters for u,m,v
! aa     = vector of parameters for simulations
! dtt    = vector of statistics for simulations
! skip   = number of samples to skip
! sample = number of networks to sample
! mtemp  = number of temperatures for simulated tempering
! qup    = probability of increasing temperature in sim tempering
! seed1  = seed for random number generation
! pr     = prob of changing a row of g
! pc     = prob of changing a column of g

! output variables
! tout   = vector of sampled network stats
! kout   = vector of temperatures sampled



!*********** ROUTINE STARTS HERE *************!

!  library
use globvar
implicit none

! input vbls and pars
integer, intent(in)::nn 
integer, intent(in)::qq
integer, intent(in)::pp(3)
real(8), dimension(pp(3)), intent(in)::aa
integer, dimension(pp(3),2), intent(in)::dtt
integer, intent(in)::skip
integer, intent(in)::sample
real(8) qup  
integer, intent(in)::mtemp 
integer, dimension(1), intent(in)::seed1
real(8), intent(in)::pr
real(8), intent(in)::pc


! output vbls and pars
real(8), dimension(pp(3),sample), intent(out)::tout
real(8), dimension(sample), intent(out)::kout

! working variables
integer i,j,k,rr,t, kk  ! indicators for loops
real(8), dimension(pp(3))::tsim2 ! current simulated network stats
integer ktemp !indicator of current temperature
real(8) invp(mtemp) ! probability of inversion for each temperature
real(8) temper(mtemp) ! vector with temperature levels
real(8) rhotemp(mtemp) ! vector of pseudopriors
real(8) rho0,nn0 ! constants for stochastic approximation of pseudopriors (see Geyer-Thompson 1995 for details)
!integer totsim  ! maximum nummber of simulations
real(8) ux,uux ! random number
real(8) odds ! odds of two networks
real(8) dtfu, dtfm, dtfv ! network statistics functions
real(8) dttemp(pp(3))


! set prob of inversion
invp(1) = 0.0 ! .01 !.01 
do k=2,mtemp
invp(k)=invp(k-1)+.05
enddo

! set temperatures
temper(1)=1
do k=2,mtemp
	temper(k)=1 + .1*(k-1)
enddo

! set pseudopriors
rhotemp(:)=100000.0
rho0 = 1.0
nn0 = 10000000.0



! set initial simulated network equal to observed one
gsim = g

! compute observed network stats
call sstat(pp,nn,qq,1,dtt,tsim)
!tsim = tobs


!** random numbers seed
!k=1
!CALL RANDOM_SEED(SIZE=k)
!CALL RANDOM_SEED(PUT=seed1)



!**************************************************!
!******* MAIN SIMULATION LOOP STARTS HERE *********!
!**************************************************!

tout(:,:)=0.0  ! initialize 
rr=1   ! sample loop indicator
ktemp =1  ! initialize temperature
totsim = 10000000 ! max number of simulations

! main loop
do t=1, totsim
    
    call random_number(uux)
    !print*, uux
    ! row step
    if (uux<pr) then
        !print*, 'row'
        
        call random_number(ux)
        i = ceiling(nn*ux) ! choose row/player i
        gsim(i,:) = 1 - gsim(i,:) !invert all links of player i
		gsim(i,i) = 0 ! make sure you did not invert the link for g_ii
        call sstat(pp,nn,qq,1,dtt,tsim2)
        odds = 0.0
        do kk=1,pp(3)
            odds = odds + aa(kk)*(tsim2(kk)-tsim(kk))
        enddo ! kk
        call random_number(ux)
        if (log(ux)<=min(0.0,odds)) then
            tsim = tsim2
            !print*, 'accepted'
		else
			gsim(i,:) = 1 - gsim(i,:) !invert all links of player i
			gsim(i,i) = 0 ! make sure you did not invert the link for g_ii

        endif ! (log(ux)<=min(0.0,odds))
    
    ! column step    
    elseif (pr <= uux .and. uux< pr+pc) then
        !print*, 'column' 
        
        call random_number(ux)
        i = ceiling(nn*ux) ! choose column i
        gsim(:,i) = 1 - gsim(:,i) !invert all links of column i
        call sstat(pp,nn,qq,1,dtt,tsim2)
        odds = 0.0
        do kk=1,pp(3)
            odds = odds + aa(kk)*(tsim2(kk)-tsim(kk))
        enddo ! kk
        call random_number(ux)
        if (log(ux)<=min(0.0,odds)) then
            tsim = tsim2
			   !print*, 'accepted'
		else
			gsim(i,:) = 1 - gsim(i,:) !invert all links of player i
			gsim(i,i) = 0 ! make sure you did not invert the link for g_ii
		
        endif ! (log(ux)<=min(0.0,odds))
        
   
    ! one link step
    !elseif (uux <= pr+pc) then
     else   
        !print*, 'link'
        
        
        ! one-link per iteration 
        call random_number(ux)
        i = ceiling(nn*ux) ! choose player i
        call random_number(ux)
        j = ceiling(nn*ux) ! choose agent j
        
        do while (i==j)
            call random_number(ux)
            j = ceiling(nn*ux) ! choose agent j
        enddo ! while loop
        
        call random_number(ux)
        odds = 0.0
        dttemp(:) = 0.0 
        
        ! direct utility update
        do kk=1,pp(1)
            dttemp(kk) = dtfu(nn,dtt(kk,1),i,j,dtt(kk,2))
        enddo
        ! mutual utility update        
        if (gsim(j,i)==1) then  ! g_ji=1 then compute mutual utility
            do kk=pp(1)+1,pp(2)
                dttemp(kk) = dtfm(nn,dtt(kk,1),i,j,dtt(kk,2))
            enddo
        endif
        ! indirect utility and popularity update
        do kk=pp(2)+1,pp(3)
            dttemp(kk) = dtfv(nn,1,dtt(kk,1),i,j,dtt(kk,2))
        enddo
        
       
        
        ! compute odds ration from the model
        do kk=1,pp(3)
            odds = odds + aa(kk)*dttemp(kk)
        enddo
        odds = odds/temper(ktemp)
        
    
    
        ! update link g_ij
        if (gsim(i,j) ==1 ) then
            if (odds<0) then
                gsim(i,j)=0 
                do kk=1,pp(3)
                    tsim(kk) = tsim(kk) - dttemp(kk)
                enddo
                
            else
                if (log(ux)<-odds) then
                   gsim(i,j)=0 
                   do kk=1,pp(3)
                        tsim(kk) = tsim(kk) - dttemp(kk)
                    enddo
                endif ! (log(ux)<-odds)
                
            endif ! (odds<0)
            
        elseif (gsim(i,j)==0) then
            if (odds>=0) then
                gsim(i,j)=1
                do kk=1,pp(3)
                    tsim(kk) = tsim(kk) + dttemp(kk)
                enddo
            else
                if (log(ux)<odds) then
                    gsim(i,j)=1
                    do kk=1,pp(3)
                        tsim(kk) = tsim(kk) + dttemp(kk)
                    enddo
                    
                endif !(log(ux)<=odds)
                
            endif !(odds>=0)
        endif !(gsim(i,j) ==1 )
          
    endif !(ux<invp(ktemp))

    
    ! sample every skip iterations
    if (rr<=sample) then
        if (t==skip*rr) then
            tout(:,rr)=tsim
            kout(rr)=ktemp
            !if (mod(rr,1000)==0) then
            !    print*, 'sample = ', rr
            !    print*, 'tsim = ', tsim
            !endif
            rr=rr+1
        endif
    else
        exit
    endif
    

    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    ! introduce simulated tempering
    
enddo ! t

end subroutine sample_ast_rc




!************************************
!* subroutine: sample_ast_rc_est
!* version: 1.0
!* date: 11/09/2012
!* last modified:  11/09/2012
!* author: Angelo Mele
!* description: Samples using Accelerated Simulated
!*              Tempering as in Li et al (2004).
!               introduces steps where row or column
!               are modified
!*******************************************

subroutine sample_ast_rc_est(nn,qq,pp,aa,dtt,skip,sample,mtemp,qup,tout,kout,seed1,pr,pc)

! input variables
! nn     = number of players
! qq     = number of variables
! pp     = number of parameters for u,m,v
! aa     = vector of parameters for simulations
! dtt    = vector of statistics for simulations
! skip   = number of samples to skip
! sample = number of networks to sample
! mtemp  = number of temperatures for simulated tempering
! qup    = probability of increasing temperature in sim tempering
! seed1  = seed for random number generation
! pr     = prob of changing a row of g
! pc     = prob of changing a column of g

! output variables
! tout   = vector of sampled network stats
! kout   = vector of temperatures sampled



!*********** ROUTINE STARTS HERE *************!

!  library
use globvar
implicit none

! input vbls and pars
integer, intent(in)::nn 
integer, intent(in)::qq
integer, intent(in)::pp(3)
real(8), dimension(pp(3)), intent(in)::aa
integer, dimension(pp(3),2), intent(in)::dtt
integer, intent(in)::skip
integer, intent(in)::sample
real(8) qup  
integer, intent(in)::mtemp 
integer, dimension(1), intent(in)::seed1
real(8), intent(in)::pr
real(8), intent(in)::pc


! output vbls and pars
real(8), dimension(pp(3)), intent(out)::tout
real(8), intent(out)::kout

! working variables
integer i,j,k,rr,t, kk  ! indicators for loops
real(8), dimension(pp(3))::tsim2 ! current simulated network stats
integer ktemp !indicator of current temperature
real(8) invp(mtemp) ! probability of inversion for each temperature
real(8) temper(mtemp) ! vector with temperature levels
real(8) rhotemp(mtemp) ! vector of pseudopriors
real(8) rho0,nn0 ! constants for stochastic approximation of pseudopriors (see Geyer-Thompson 1995 for details)
!integer totsim  ! maximum nummber of simulations
real(8) ux,uux ! random number
real(8) odds ! odds of two networks
real(8) dtfu, dtfm, dtfv ! network statistics functions
real(8) dttemp(pp(3))


! set prob of inversion
invp(1) = 0.0 ! .01 !.01 
do k=2,mtemp
invp(k)=invp(k-1)+.05
enddo

! set temperatures
temper(1)=1
do k=2,mtemp
	temper(k)=1 + .1*(k-1)
enddo

! set pseudopriors
rhotemp(:)=100000.0
rho0 = 1.0
nn0 = 10000000.0



! set initial simulated network equal to observed one
gsim = g

! compute observed network stats
call sstat(pp,nn,qq,1,dtt,tsim)
!tsim = tobs


!** random numbers seed
!k=1
!CALL RANDOM_SEED(SIZE=k)
!CALL RANDOM_SEED(PUT=seed1)



!**************************************************!
!******* MAIN SIMULATION LOOP STARTS HERE *********!
!**************************************************!

tout(:)=0.0  ! initialize 
rr=1   ! sample loop indicator
ktemp =1  ! initialize temperature
totsim = 10000000 ! max number of simulations

! main loop
do t=1, totsim
    
    call random_number(uux)
    !print*, uux
    ! row step
    if (uux<pr) then
        !print*, 'row'
        
        call random_number(ux)
        i = ceiling(nn*ux) ! choose row/player i
        gsim(i,:) = 1 - gsim(i,:) !invert all links of player i
		gsim(i,i) = 0 ! make sure you did not invert the link for g_ii
        call sstat(pp,nn,qq,1,dtt,tsim2)
        odds = 0.0
        do kk=1,pp(3)
            odds = odds + aa(kk)*(tsim2(kk)-tsim(kk))
        enddo ! kk
        call random_number(ux)
        if (log(ux)<=min(0.0,odds)) then
            tsim = tsim2
            !print*, 'accepted'
		else
			gsim(i,:) = 1 - gsim(i,:) !invert all links of player i
			gsim(i,i) = 0 ! make sure you did not invert the link for g_ii

        endif ! (log(ux)<=min(0.0,odds))
    
    ! column step    
    elseif (pr <= uux .and. uux< pr+pc) then
        !print*, 'column' 
        
        call random_number(ux)
        i = ceiling(nn*ux) ! choose column i
        gsim(:,i) = 1 - gsim(:,i) !invert all links of column i
        call sstat(pp,nn,qq,1,dtt,tsim2)
        odds = 0.0
        do kk=1,pp(3)
            odds = odds + aa(kk)*(tsim2(kk)-tsim(kk))
        enddo ! kk
        call random_number(ux)
        if (log(ux)<=min(0.0,odds)) then
            tsim = tsim2
			   !print*, 'accepted'
		else
			gsim(i,:) = 1 - gsim(i,:) !invert all links of player i
			gsim(i,i) = 0 ! make sure you did not invert the link for g_ii
		
        endif ! (log(ux)<=min(0.0,odds))
        
   
    ! one link step
    !elseif (uux <= pr+pc) then
     else   
        !print*, 'link'
        
        
        ! one-link per iteration 
        call random_number(ux)
        i = ceiling(nn*ux) ! choose player i
        call random_number(ux)
        j = ceiling(nn*ux) ! choose agent j
        
        do while (i==j)
            call random_number(ux)
            j = ceiling(nn*ux) ! choose agent j
        enddo ! while loop
        
        call random_number(ux)
        odds = 0.0
        dttemp(:) = 0.0 
        
        ! direct utility update
        do kk=1,pp(1)
            dttemp(kk) = dtfu(nn,dtt(kk,1),i,j,dtt(kk,2))
        enddo
        ! mutual utility update        
        if (gsim(j,i)==1) then  ! g_ji=1 then compute mutual utility
            do kk=pp(1)+1,pp(2)
                dttemp(kk) = dtfm(nn,dtt(kk,1),i,j,dtt(kk,2))
            enddo
        endif
        ! indirect utility and popularity update
        do kk=pp(2)+1,pp(3)
            dttemp(kk) = dtfv(nn,1,dtt(kk,1),i,j,dtt(kk,2))
        enddo
        
       
        
        ! compute odds ration from the model
        do kk=1,pp(3)
            odds = odds + aa(kk)*dttemp(kk)
        enddo
        odds = odds/temper(ktemp)
        
    
    
        ! update link g_ij
        if (gsim(i,j) ==1 ) then
            if (odds<0) then
                gsim(i,j)=0 
                do kk=1,pp(3)
                    tsim(kk) = tsim(kk) - dttemp(kk)
                enddo
                
            else
                if (log(ux)<-odds) then
                   gsim(i,j)=0 
                   do kk=1,pp(3)
                        tsim(kk) = tsim(kk) - dttemp(kk)
                    enddo
                endif ! (log(ux)<-odds)
                
            endif ! (odds<0)
            
        elseif (gsim(i,j)==0) then
            if (odds>=0) then
                gsim(i,j)=1
                do kk=1,pp(3)
                    tsim(kk) = tsim(kk) + dttemp(kk)
                enddo
            else
                if (log(ux)<odds) then
                    gsim(i,j)=1
                    do kk=1,pp(3)
                        tsim(kk) = tsim(kk) + dttemp(kk)
                    enddo
                    
                endif !(log(ux)<=odds)
                
            endif !(odds>=0)
        endif !(gsim(i,j) ==1 )
          
    endif !(ux<invp(ktemp))

    
    ! sample every skip iterations
    if (rr<=sample) then
        if (t==skip*rr) then
            !tout(:,rr)=tsim
            !kout(rr)=ktemp
            !if (mod(rr,1000)==0) then
            !    print*, 'sample = ', rr
            !    print*, 'tsim = ', tsim
            !endif
            rr=rr+1
        endif
    else
        exit
    endif
    

    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    ! introduce simulated tempering
    
enddo ! t

tout = tsim
kout = 1

end subroutine sample_ast_rc_est







!************************************
!* subroutine: bayesest_ex
!* version: 1.0
!* date: 03/05/2010
!* last modified: 01/10/2012
!* author: Angelo Mele
!* description: samples parameters from the model
!*              using an Exchange algorithm (Friel and Caimo 2010)
!*              Accelerated Simulated Tempering
!*              algorithm. writes the sampled parameters to txt file
!************************************

!subroutine bayesest_ex(nn,pp,qq,dt,burnin,skip,parsamp,samp,step,,seed1)
!subroutine bayesest_ex(nn,pp,qq,dt,burnin,skip,parsamp,samp,step,mtempnet,qup,seed1,pr,pc)
subroutine bayesest_ex(nn,pp,qq,dt,skip,parsamp,samp,step,mtempnet,qup,seed1,pr,pc)
use globvar

! input vbls
integer, intent(in)::nn  ! size of network
integer, intent(in)::pp(3)  ! parameter structure
integer, intent(in)::qq   ! number of covariates
!integer, intent(in):: burnin  ! burnin for mcmc networks
integer, intent(in)::skip  ! networks to skip in sampling
integer, dimension(pp(3),2),intent(in)::dt  ! type of covariates matrix
integer, intent(in):: parsamp  ! size of the sample for parameters
integer, intent(in):: samp  ! size of the sample for networks
real(8), dimension(pp(3),pp(3)), intent(in):: step  ! cholesky matrix for proposal 
integer, intent(in):: mtempnet ! number of temperatures of the AST for networks
real(8), intent(in):: qup ! probability of going up one temp
integer, dimension(1), intent(in)::seed1
real(8), intent(in)::pr,pc ! prob of inverting row or column for network simulation

! working vbls
real(8), dimension(pp(3)):: tsamp  ! last networks from the network simulation
!real(8), dimension(pp(3),samp)::tout !sample of networks from the network simulation
!real(8), dimension(samp)::kout ! sample temperatures from network simulation
real(8) kout ! sample temperatures from network simulation
real(8), dimension(pp(3),pp(3)):: stderr ! var cov from networks samples (not needed)
integer t,i,r ! counters for loops
real(8) ux, pmh
real(8), dimension(pp(3)):: uux ! vector of random variables for parameter update
real(8), dimension(pp(3)):: thetap ! proposed parameter vector 
real(8), dimension(pp(3)):: theta  ! current parameter vector
integer nacc, nrej  ! number of acceptances and rejections for updates
real(8) racc ! ratio of acceptances
!real(8), dimension(mtemp):: rhotemp
!real(8), dimension(mtemp):: temper
real(8) qdown
real(8) rst
real(8) normal
!real(8), dimension(samp)::odds
real(8) maxodds
real(8) consratio ! ratio of constants
real(8) mu0,sigma0, mu1,sigma1  ! parameters for update
real(8), dimension(pp(3))::tobs2



mu0=0.0
sigma0=3.0
mu1=0.0
sigma1=1.0




!** random numbers seed
k=1
!CALL RANDOM_SEED(SIZE=k)
!CALL RANDOM_SEED(PUT=seed1)
CALL RANDOM_SEED()


!**** initialize variables
do i=1,pp(3)
	theta(i) = normal(mu1,step(i,i))
	!theta(i) = normal(mu1,sigma1)
enddo



!theta = (/-20.0, 15.0/) !


!open(1,file='parbayes_exchange3.txt')
!open(2,file='accratio_exchange3.txt')
!open(3,file='c:/SpatSocNet/programs/parallel/expfeb2010/thetaseq.txt')


!**** start main loop
t = 1
racc = 0.0
nacc = 0
nrej = 0
do while (t<=parsamp)
	
	! print current stuff
	!if (mod(t,100)==0) then
	!	print*, 'iter = ', t
	!endif


	
	!**** propose a new theta
	do i=1,pp(3)
		uux(i)=normal(mu1,sigma1)  ! random number for random walk update
	enddo
	!print*, 'uux '
	!print*, uux
	do i=1,pp(3)
		thetap(i) = theta(i) + dot_product(step(i,:),uux) ! propose new element i of parameter vector
		!print*, 'dot_product(step(i,:),uux)', dot_product(step(i,:),uux)
	enddo !i  

    !if (mod(t,100)==0) then
    !    print*, 'current theta vs proposal'
!	    print*, 'ITERATION = ', t
!	    print*, 'TEMPERATURE = ', ktemp
!	    print*, 'ACCEPTED =', racc
!	    do i=1,pp(3)
!		    print*, theta(i),thetap(i)
!	    enddo
!    endif


	!**** collect sample to compute constant ratio
	!*******************************************************************
	!**** call seed_random_number()   !**** 5/6/2011
	!*******************************************************************
	
	!call sample_ast_exp_ex(nn,qq,pp,thetap,dt,samp,mtempnet,tsamp,seed1)
    !call sample_ast_rc(nn,qq,pp,thetap,dt,skip,samp,mtempnet,qup,tout,kout,seed1,pr,pc)
    call sample_ast_rc_est(nn,qq,pp,thetap,dt,skip,samp,mtempnet,qup,tsamp,kout,seed1,pr,pc)
    !tsamp = tout(:,samp)
	
!    if (mod(t,100)==0) then
!	    print*, ''
!	    print*, 'tsamp = ', tsamp
!	    print*, 'tobs  = ', tobs
!	    print*, ''
 !   endif
    ! check for tsim 
    !call sstat(pp,nn,qq,nnz,isim,jsim,x,dt,tobs2)
    !print*, 'tobs2=', tobs2

	!**** consider only sample from low temp
	!if (ktemp==1) then


    !**** accept new parameter using AST approximate ratio
	!pmh = dot_product(thetap-theta,(tsamp-tobs)/temper(ktemp))  - .5*sum(thetap**2-theta**2)/sigma0
	!pmh = dot_product(theta-thetap,(tsamp-tobs)/temper(ktemp))  - .5*sum(thetap**2-theta**2)/sigma0
	pmh = dot_product(theta-thetap,(tsamp-tobs))  - .5*sum(thetap**2-theta**2)/sigma0
	write(2,*) pmh
	if (pmh>0.0) then
		theta = thetap
		write(1,"(200f16.10)") theta
		nacc=nacc+1
		racc = 1.0*nacc
		racc = racc/t
		write(2,*) racc

	!**** if fp smaller than current fcn, then use MH to update
	else 
		call random_number(ux)
		if (log(ux)<=pmh) then
			theta = thetap
			write(1,"(200f16.10)") theta
			nacc  = nacc + 1
			racc = 1.0*nacc
			racc = racc/t
			write(2,*) racc
		else
			nrej  = nrej + 1
			write(1,"(200f16.10)") theta
			racc = 1.0*nacc
			racc = racc/t
			write(2,*) racc
		endif
	
	endif

	t = t + 1
	!endif
	



	! update temperature
	


	!! update temperature if AST
	!if(mtemp>1) then
 !
	!if (ktemp==1) then
	!	rst  = min(0.0,log(rhotemp(2)/rhotemp(1))+dot_product(theta,tobs)*(1/temper(2)-1/temper(1))+log(qup/1))
 !
	!	call random_number(ux)
	!	if (log(ux)<=rst) then
	!		ktemp=2
	!	endif !(log(ux)<=rst)
	!elseif (ktemp==mtemp) then
	!	rst  = min(0.0,log(rhotemp(mtemp-1)/rhotemp(mtemp))+dot_product(theta,tobs)*(1/temper(mtemp-1)-1/temper(mtemp))+log(1/qdown))
	!	call random_number(ux)
	!	if (log(ux)<=rst) then
	!		ktemp=mtemp-1
	!	endif !(log(ux)<=rst)
	!else 
	!	call random_number(ux)
	!	if (ux<=qup) then
	!		rst  = min(0.0,log(rhotemp(ktemp+1)/rhotemp(ktemp))+dot_product(theta,tobs)*(1/temper(ktemp+1)-1/temper(ktemp))+log(qdown/qup))
	!		call random_number(ux)
	!		if (log(ux)<=rst) then
	!			ktemp=ktemp+1
	!		endif !(log(ux)<=rst)
	!	else
	!		rst  = min(0.0,log(rhotemp(ktemp-1)/rhotemp(ktemp))+dot_product(theta,tobs)*(1/temper(ktemp-1)-1/temper(ktemp))+log(qup/qdown))
	!		call random_number(ux)
	!		if (log(ux)<=rst) then
	!			ktemp=ktemp-1
	!		endif !(log(ux)<=rst)		
	!	endif !(ux=<qup)
	!endif	
 !
	!endif		
	
enddo ! t main loop




close(1)
close(2)

end subroutine bayesest_ex










!************************************
!* subroutine: bayes_mh_erdos
!* version: 1.0
!* date: 03/05/2010
!* last modified: 01/10/2012
!* author: Angelo Mele
!* description: samples parameters from the posterior of an erdos renyi model
!*              using a Metropolis-Hastings
!*              algorithm. writes the sampled parameters to txt file
!************************************

!subroutine bayesest_ex(nn,pp,qq,dt,burnin,skip,parsamp,samp,step,,seed1)
!subroutine bayesest_ex(nn,pp,qq,dt,burnin,skip,parsamp,samp,step,mtempnet,qup,seed1,pr,pc)
subroutine bayes_mh_erdos(nn,pp,qq,dt,parsamp,step, seed1)

use globvar

! input vbls
integer, intent(in)::nn  ! size of network
integer, intent(in)::pp(3)  ! parameter structure
integer, intent(in)::qq   ! number of covariates
integer, dimension(pp(3),2),intent(in)::dt  ! type of covariates matrix
integer, intent(in):: parsamp  ! size of the sample for parameters
real(8), dimension(pp(3),pp(3)), intent(in):: step  ! cholesky matrix for proposal 
integer, dimension(1), intent(in)::seed1

! working vbls
integer t,i,r ! counters for loops
real(8) ux, pmh
real(8), dimension(pp(3)):: uux ! vector of random variables for parameter update
real(8), dimension(pp(3)):: thetap ! proposed parameter vector 
real(8), dimension(pp(3)):: theta  ! current parameter vector
integer nacc, nrej  ! number of acceptances and rejections for updates
real(8) racc ! ratio of acceptances
real(8) rst
real(8) normal

real(8) mu0,sigma0, mu1,sigma1  ! parameters for update





mu0=0.0
sigma0=3.0
mu1=0.0
sigma1=1.0




!** random numbers seed
k=1
CALL RANDOM_SEED(SIZE=k)
CALL RANDOM_SEED(PUT=seed1)


!**** initialize variables
do i=1,pp(3)
	theta(i) = normal(mu1,step(i,i))
	!theta(i) = normal(mu1,sigma1)
enddo



!theta = (/30.0,-2.0,-5.0,4.0,.01,.2/)
!theta= (/-2.238967, 0.6881534, -0.1906898, 0.6725112, 0.9393955, -0.002182496/)


! ******* intitial values for convergence experiments *******
theta = -2.0
!theta = (/-2.0, .5, .01/) ! (1)
!theta = (/-10.0, 5.0, 1.0/) ! (2)
!theta = (/10.0, -5.0, -1.0/) ! (3)
!theta = (/-3.0, -.05, .3/) ! (4)
!theta = (/-20.0, 15.0, -3.0/) ! (5)


!open(1,file='parbayes_mh_erdos.txt')
!open(2,file='accratio_mh_erdos.txt')
!open(3,file='c:/SpatSocNet/programs/parallel/expfeb2010/thetaseq.txt')


!**** start main loop
t = 1
racc = 0.0
nacc = 0
nrej = 0
do while (t<=parsamp)
	
	! print current stuff
!	if (mod(t,100)==0) then
!		print*, 'iter = ', t
!	endif


	
	!**** propose a new theta
	do i=1,pp(3)
		uux(i)=normal(mu1,sigma1)  ! random number for random walk update
	enddo
	!print*, 'uux '
	!print*, uux
	do i=1,pp(3)
		thetap(i) = theta(i) + dot_product(step(i,:),uux) ! propose new element i of parameter vector
		!print*, 'dot_product(step(i,:),uux)', dot_product(step(i,:),uux)
	enddo !i  

!	if (mod(t,1000)==0) then
!        print*, 'current theta vs proposal'
!	    print*, 'ITERATION = ', t
!	    print*, 'TEMPERATURE = ', ktemp
!	    print*, 'ACCEPTED =', racc
!	    do i=1,pp(3)
!    		print*, theta(i),thetap(i)
 !   	enddo
!    endif


    !**** accept new parameter using AST approximate ratio
	!pmh = dot_product(thetap-theta,(tsamp-tobs)/temper(ktemp))  - .5*sum(thetap**2-theta**2)/sigma0
	!pmh = dot_product(theta-thetap,(tsamp-tobs)/temper(ktemp))  - .5*sum(thetap**2-theta**2)/sigma0
	pmh = dot_product(thetap-theta,tobs) + nn*(nn-1)*log((1+exp(theta(1))) / (1+exp(thetap(1)))) - .5*sum(thetap**2-theta**2)/sigma0
	write(2,*) pmh
	if (pmh>0.0) then
		theta = thetap
		write(1,"(200f16.10)") theta
		nacc=nacc+1
		racc = 1.0*nacc
		racc = racc/t
		write(2,*) racc

	!**** if fp smaller than current fcn, then use MH to update
	else 
		call random_number(ux)
		if (log(ux)<=pmh) then
			theta = thetap
			write(1,"(200f16.10)") theta
			nacc  = nacc + 1
			racc = 1.0*nacc
			racc = racc/t
			write(2,*) racc
		else
			nrej  = nrej + 1
			write(1,"(200f16.10)") theta
			racc = 1.0*nacc
			racc = racc/t
			write(2,*) racc
		endif
	
	endif

	t = t + 1
	!endif
	



	! update temperature
	


	!! update temperature if AST
	!if(mtemp>1) then
 !
	!if (ktemp==1) then
	!	rst  = min(0.0,log(rhotemp(2)/rhotemp(1))+dot_product(theta,tobs)*(1/temper(2)-1/temper(1))+log(qup/1))
 !
	!	call random_number(ux)
	!	if (log(ux)<=rst) then
	!		ktemp=2
	!	endif !(log(ux)<=rst)
	!elseif (ktemp==mtemp) then
	!	rst  = min(0.0,log(rhotemp(mtemp-1)/rhotemp(mtemp))+dot_product(theta,tobs)*(1/temper(mtemp-1)-1/temper(mtemp))+log(1/qdown))
	!	call random_number(ux)
	!	if (log(ux)<=rst) then
	!		ktemp=mtemp-1
	!	endif !(log(ux)<=rst)
	!else 
	!	call random_number(ux)
	!	if (ux<=qup) then
	!		rst  = min(0.0,log(rhotemp(ktemp+1)/rhotemp(ktemp))+dot_product(theta,tobs)*(1/temper(ktemp+1)-1/temper(ktemp))+log(qdown/qup))
	!		call random_number(ux)
	!		if (log(ux)<=rst) then
	!			ktemp=ktemp+1
	!		endif !(log(ux)<=rst)
	!	else
	!		rst  = min(0.0,log(rhotemp(ktemp-1)/rhotemp(ktemp))+dot_product(theta,tobs)*(1/temper(ktemp-1)-1/temper(ktemp))+log(qup/qdown))
	!		call random_number(ux)
	!		if (log(ux)<=rst) then
	!			ktemp=ktemp-1
	!		endif !(log(ux)<=rst)		
	!	endif !(ux=<qup)
	!endif	
 !
	!endif		
	
enddo ! t main loop




close(1)
close(2)

end subroutine bayes_mh_erdos







!************************************
!* subroutine: bayes_mh_recip
!* version: 1.0
!* date: 03/05/2010
!* last modified: 01/10/2012
!* author: Angelo Mele
!* description: samples parameters from the posterior of a 2 pars, reciprocity model
!*              using a Metropolis-Hastings
!*              algorithm. writes the sampled parameters to txt file
!************************************

!subroutine bayesest_ex(nn,pp,qq,dt,burnin,skip,parsamp,samp,step,,seed1)
!subroutine bayesest_ex(nn,pp,qq,dt,burnin,skip,parsamp,samp,step,mtempnet,qup,seed1,pr,pc)
subroutine bayes_mh_recip(nn,pp,qq,dt,parsamp,step, seed1)

use globvar

! input vbls
integer, intent(in)::nn  ! size of network
integer, intent(in)::pp(3)  ! parameter structure
integer, intent(in)::qq   ! number of covariates
integer, dimension(pp(3),2),intent(in)::dt  ! type of covariates matrix
integer, intent(in):: parsamp  ! size of the sample for parameters
real(8), dimension(pp(3),pp(3)), intent(in):: step  ! cholesky matrix for proposal 
integer, dimension(1), intent(in)::seed1

! working vbls
integer t,i,r ! counters for loops
real(8) ux, pmh
real(8), dimension(pp(3)):: uux ! vector of random variables for parameter update
real(8), dimension(pp(3)):: thetap ! proposed parameter vector 
real(8), dimension(pp(3)):: theta  ! current parameter vector
integer nacc, nrej  ! number of acceptances and rejections for updates
real(8) racc ! ratio of acceptances
real(8) rst
real(8) normal

real(8) mu0,sigma0, mu1,sigma1  ! parameters for update





mu0=0.0
sigma0=3.0
mu1=0.0
sigma1=1.0




!** random numbers seed
k=1
!CALL RANDOM_SEED(SIZE=k)
!CALL RANDOM_SEED(PUT=seed1)
CALL RANDOM_SEED()

!**** initialize variables
do i=1,pp(3)
	theta(i) = normal(mu1,step(i,i))
	!theta(i) = normal(mu1,sigma1)
enddo



!theta = (/30.0,-2.0,-5.0,4.0,.01,.2/)
!theta= (/-2.238967, 0.6881534, -0.1906898, 0.6725112, 0.9393955, -0.002182496/)


! ******* intitial values for convergence experiments *******
!theta = (/-2.0, 0.5/)
!theta = (/-2.0, .5, .01/) ! (1)
!theta = (/-10.0, 5.0/) ! (2)
!theta = (/10.0, -5.0/) ! (3)
!theta = (/-3.0, -.05/) ! (4)
theta = (/-20.0, 5.0, -15.0, 0.3/) ! (5)


!open(1,file='parbayes_mh_erdos.txt')
!open(2,file='accratio_mh_erdos.txt')
!open(3,file='c:/SpatSocNet/programs/parallel/expfeb2010/thetaseq.txt')


!**** start main loop
t = 1
racc = 0.0
nacc = 0
nrej = 0
do while (t<=parsamp)
	
	! print current stuff
!	if (mod(t,100)==0) then
!		print*, 'iter = ', t
!	endif


	
	!**** propose a new theta
	do i=1,pp(3)
		uux(i)=normal(mu1,sigma1)  ! random number for random walk update
	enddo
	!print*, 'uux '
	!print*, uux
	do i=1,pp(3)
		thetap(i) = theta(i) + dot_product(step(i,:),uux) ! propose new element i of parameter vector
		!print*, 'dot_product(step(i,:),uux)', dot_product(step(i,:),uux)
	enddo !i  

!	if (mod(t,100)==0) then
!        print*, 'current theta vs proposal'
!	    print*, 'ITERATION = ', t
!	    print*, 'TEMPERATURE = ', ktemp
!	    print*, 'ACCEPTED =', racc
!	    do i=1,pp(3)
!    		print*, theta(i),thetap(i)
!    	enddo
!    endif


    !**** accept new parameter using AST approximate ratio
	!pmh = dot_product(thetap-theta,(tsamp-tobs)/temper(ktemp))  - .5*sum(thetap**2-theta**2)/sigma0
	!pmh = dot_product(theta-thetap,(tsamp-tobs)/temper(ktemp))  - .5*sum(thetap**2-theta**2)/sigma0
	pmh = dot_product(thetap-theta,tobs) 
	pmh = pmh + .5*nn*(nn-1)*log( (1+2*exp(theta(1))+exp(2*theta(1)+theta(2)) ) / (1+2*exp(thetap(1))+exp(2*thetap(1)+thetap(2)) ) ) 
	pmh = pmh  - .5*sum(thetap**2-theta**2)/sigma0
	write(2,*) pmh
	if (pmh>0.0) then
		theta = thetap
		write(1,"(200f16.10)") theta
		nacc=nacc+1
		racc = 1.0*nacc
		racc = racc/t
		write(2,*) racc

	!**** if fp smaller than current fcn, then use MH to update
	else 
		call random_number(ux)
		if (log(ux)<=pmh) then
			theta = thetap
			write(1,"(200f16.10)") theta
			nacc  = nacc + 1
			racc = 1.0*nacc
			racc = racc/t
			write(2,*) racc
		else
			nrej  = nrej + 1
			write(1,"(200f16.10)") theta
			racc = 1.0*nacc
			racc = racc/t
			write(2,*) racc
		endif
	
	endif

	t = t + 1
	!endif
	



	! update temperature
	


	!! update temperature if AST
	!if(mtemp>1) then
 !
	!if (ktemp==1) then
	!	rst  = min(0.0,log(rhotemp(2)/rhotemp(1))+dot_product(theta,tobs)*(1/temper(2)-1/temper(1))+log(qup/1))
 !
	!	call random_number(ux)
	!	if (log(ux)<=rst) then
	!		ktemp=2
	!	endif !(log(ux)<=rst)
	!elseif (ktemp==mtemp) then
	!	rst  = min(0.0,log(rhotemp(mtemp-1)/rhotemp(mtemp))+dot_product(theta,tobs)*(1/temper(mtemp-1)-1/temper(mtemp))+log(1/qdown))
	!	call random_number(ux)
	!	if (log(ux)<=rst) then
	!		ktemp=mtemp-1
	!	endif !(log(ux)<=rst)
	!else 
	!	call random_number(ux)
	!	if (ux<=qup) then
	!		rst  = min(0.0,log(rhotemp(ktemp+1)/rhotemp(ktemp))+dot_product(theta,tobs)*(1/temper(ktemp+1)-1/temper(ktemp))+log(qdown/qup))
	!		call random_number(ux)
	!		if (log(ux)<=rst) then
	!			ktemp=ktemp+1
	!		endif !(log(ux)<=rst)
	!	else
	!		rst  = min(0.0,log(rhotemp(ktemp-1)/rhotemp(ktemp))+dot_product(theta,tobs)*(1/temper(ktemp-1)-1/temper(ktemp))+log(qup/qdown))
	!		call random_number(ux)
	!		if (log(ux)<=rst) then
	!			ktemp=ktemp-1
	!		endif !(log(ux)<=rst)		
	!	endif !(ux=<qup)
	!endif	
 !
	!endif		
	
enddo ! t main loop




close(1)
close(2)

end subroutine bayes_mh_recip





function normal(mean,sigma) !returns a normal distribution 

implicit none 
real(8) normal,tmp 
real(8), intent(in)::mean,sigma 
integer flag 
real(8) fac,gsave,rsq,r1,r2 
save flag,gsave 
data flag /0/ 
	if (flag.eq.0) then 
		rsq=2.0
		do while(rsq.ge.1.0.or.rsq.eq.0.0) ! new from for do 
			call random_number(r1)
			call random_number(r2)
			r1=2.0*r1-1.0
			r2=2.0*r2-1.0 
			rsq=r1*r1+r2*r2 
		enddo 
		fac=sqrt(-2.0*log(rsq)/rsq) 
		gsave=r1*fac 
		tmp=r2*fac 
		flag=1 
	else 
		tmp=gsave 
		flag=0 
	endif 
normal=tmp*sigma+mean 
return 
end function normal
